package ubiik.utils.view;

import java.util.LinkedList;
import java.util.List;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;

public class InputHistoryHandler {
  private static final int MAX_HISTORY_DEPTH = 100;
  
  private TextField textFieldInput;
  
  private List<String> commandsHistory;
  
  private int commandsHistoryPos;
  
  public InputHistoryHandler(TextField textField) {
    this.textFieldInput = textField;
    this.commandsHistory = new LinkedList<>();
    this.commandsHistoryPos = -1;
    this.textFieldInput.addEventFilter(KeyEvent.KEY_PRESSED, e -> {
          boolean isKeyUp = e.getCode().equals(KeyCode.UP);
          boolean isKeyDown = e.getCode().equals(KeyCode.DOWN);
          if (isKeyUp || isKeyDown) {
            if (isKeyUp) {
              if (this.commandsHistoryPos < this.commandsHistory.size() - 1)
                this.commandsHistoryPos++; 
            } else if (isKeyDown && this.commandsHistoryPos >= 0) {
              this.commandsHistoryPos--;
            } 
            if (this.commandsHistoryPos < 0) {
              this.textFieldInput.setText("");
            } else {
              this.textFieldInput.setText(this.commandsHistory.get(this.commandsHistoryPos));
            } 
            this.textFieldInput.end();
            e.consume();
          } 
        });
  }
  
  public void resetPosition() {
    this.commandsHistoryPos = -1;
  }
  
  public void addCommandToHistory(String cmd) {
    this.commandsHistory.add(0, cmd);
    if (this.commandsHistory.size() > 100)
      this.commandsHistory.remove(this.commandsHistory.size() - 1); 
  }
}
