package ubiik.utils.view;

import java.util.Optional;
import javafx.application.Platform;
import javafx.css.PseudoClass;
import javafx.scene.control.Alert;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;

public class ValidationView {
  public static void setTextFieldHasError(TextField textField, boolean hasError) {
    PseudoClass errorClass = PseudoClass.getPseudoClass("error");
    textField.pseudoClassStateChanged(errorClass, hasError);
  }
  
  public static void setTextFieldIsSuccessful(TextField textField, boolean successful) {
    PseudoClass errorClass = PseudoClass.getPseudoClass("success");
    textField.pseudoClassStateChanged(errorClass, successful);
  }
  
  public static void setTextFieldClean(TextField textField) {
    setTextFieldHasError(textField, false);
    setTextFieldIsSuccessful(textField, false);
  }
  
  public static void displayErrorMessage(final String title, final String error) {
    Platform.runLater(new Runnable() {
          public void run() {
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle(title);
            alert.setContentText(error);
            alert.showAndWait();
          }
        });
  }
  
  public static Integer validateAndGetIntegerTextFieldMinMax(TextField textField, int min, int max) {
    boolean error = false;
    int value = -1;
    try {
      value = Integer.parseInt(textField.getText());
      if (value < min || value > max)
        error = true; 
    } catch (Exception e) {
      error = true;
    } 
    setTextFieldHasError(textField, error);
    if (!error)
      return Integer.valueOf(value); 
    return null;
  }
  
  public static void updateLabelTextUIThread(final Label label, final String text) {
    Platform.runLater(new Runnable() {
          public void run() {
            label.setText(text);
          }
        });
  }
  
  public static boolean displayConfirmationDialogAndGetResult(String title, String header, String content) {
    boolean accept = false;
    Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
    alert.setTitle(title);
    alert.setHeaderText(header);
    alert.setContentText(content);
    Optional<ButtonType> result = alert.showAndWait();
    if (result.get() == ButtonType.OK)
      accept = true; 
    return accept;
  }
}
