package ubiik.wp.admin.model.fpga;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import ubiik.wp.model.fpga.FPGARegister;
import ubiik.wp.model.fpga.FPGARegisterFactory;
import ubiik.wp.model.fpga.FPGARegisterField;
import ubiik.wp.util.ApplicationLogger;

public final class AdminFPGARegisterFactory extends FPGARegisterFactory {
  public ArrayList<FPGARegister> loadRegisters() {
    ApplicationLogger.debug("Load FPGA Registers");
    ArrayList<FPGARegister> registers = new ArrayList<>();
    InputStream in = AdminFPGARegisterFactory.class.getResourceAsStream("/main/resources/fpga_register_config.json");
    JSONParser jsonParser = new JSONParser();
    try {
      JSONObject root = (JSONObject)jsonParser.parse(new InputStreamReader(in, "UTF-8"));
      JSONArray registersArray = (JSONArray)root.get("parameters");
      ApplicationLogger.debug("Total FPGA Registers: " + registersArray.size());
      for (Object object : registersArray) {
        try {
          JSONObject jObject = (JSONObject)object;
          Long idx = (Long)jObject.get("index");
          FPGARegister register = new FPGARegister(idx.intValue());
          JSONArray fields = (JSONArray)jObject.get("fields");
          for (Object objField : fields) {
            try {
              JSONObject jObjectField = (JSONObject)objField;
              String name = (String)jObjectField.get("name");
              name = (name != null) ? name : "";
              Long msb = (Long)jObjectField.get("msb");
              Long lsb = (Long)jObjectField.get("lsb");
              String defaultValueStr = (String)jObjectField.get("default");
              defaultValueStr = (defaultValueStr != null) ? defaultValueStr : "00000000";
              String description = (String)jObjectField.get("description");
              description = (description != null) ? description : "";
              boolean writable = (jObjectField.get("write") != null) ? ((Boolean)jObjectField.get("write")).booleanValue() : false;
              if (defaultValueStr.length() < 8) {
                defaultValueStr = (new String(new char[8 - defaultValueStr.length()])).replace("\000", "0") + (new String(new char[8 - defaultValueStr.length()])).replace("\000", "0");
              } else {
                defaultValueStr = defaultValueStr.substring(0, 8);
              }
              int defaultValue = Integer.parseUnsignedInt(defaultValueStr, 16);
              FPGARegisterField field = new FPGARegisterField(register, name, msb.shortValue(), lsb.shortValue(), defaultValue, defaultValue, description, writable);
              register.addField(field);
            } catch (Exception e) {
              System.err.println("Error parsing field");
              e.printStackTrace();
            }
          }
          registers.add(register);
        } catch (Exception e) {
          System.err.println("Error parsing register");
        }
      }
    } catch (IOException|org.json.simple.parser.ParseException e) {
      e.printStackTrace();
    }
    return registers;
  }
}
