package ubiik.wp.admin.model.sdr;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import ubiik.wp.model.sdr.SDRParameter;
import ubiik.wp.model.sdr.SDRParameterFactory;
import ubiik.wp.model.sdr.SDRParameterUInt32;
import ubiik.wp.model.sdr.SDRParameterUint8;
import ubiik.wp.model.sdr.SDRparameterDouble;

public final class AdminSDRParameterFactory extends SDRParameterFactory {
  public ArrayList<SDRParameter> loadSDRParameters() {
    ArrayList<SDRParameter> parameters = new ArrayList<>();
    InputStream in = AdminSDRParameterFactory.class.getResourceAsStream("/main/resources/sdr_config.json");
    JSONParser jsonParser = new JSONParser();
    try {
      JSONObject root = (JSONObject)jsonParser.parse(new InputStreamReader(in, "UTF-8"));
      JSONArray parametersArray = (JSONArray)root.get("parameters");
      for (Object object : parametersArray) {
        try {
          SDRParameterUInt32 sDRParameterUInt32 = null;
          JSONObject jObject = (JSONObject)object;
          Long id = (Long)jObject.get("id");
          String paramType = (String)jObject.get("type");
          String name = (String)jObject.get("name");
          name = (name != null) ? name : "";
          String category = (String)jObject.get("category");
          category = (category != null) ? category : "";
          String description = (String)jObject.get("description");
          description = (description != null) ? description : "";
          Double min = (Double)jObject.get("min");
          Double max = (Double)jObject.get("max");
          Double defaultValue = (Double)jObject.get("default");
          Double step = (Double)jObject.get("step");
          step = Double.valueOf((step != null) ? step.doubleValue() : 0.0D);
          SDRParameter parameter = null;
          if (id != null && paramType != null) {
            SDRparameterDouble sDRparameterDouble;
            SDRParameterUint8 sDRParameterUint8;
            switch (paramType) {
              case "double":
                min = Double.valueOf((min != null) ? min.doubleValue() : Double.MIN_VALUE);
                max = Double.valueOf((max != null) ? max.doubleValue() : Double.MAX_VALUE);
                defaultValue = (defaultValue != null) ? defaultValue : min;
                sDRparameterDouble = new SDRparameterDouble(id.intValue(), name, description, category, defaultValue.doubleValue(), defaultValue.doubleValue(), max.doubleValue(), min.doubleValue(), step.doubleValue());
                break;
              case "uint8":
                min = Double.valueOf((min != null) ? min.doubleValue() : 0.0D);
                max = Double.valueOf((max != null) ? max.doubleValue() : 255.0D);
                defaultValue = (defaultValue != null) ? defaultValue : min;
                sDRParameterUint8 = new SDRParameterUint8(id.intValue(), name, description, category, defaultValue.intValue(), defaultValue.intValue(), max.intValue(), min.intValue(), step.intValue());
                break;
              case "int":
                min = Double.valueOf((min != null) ? min.doubleValue() : -2.147483648E9D);
                max = Double.valueOf((max != null) ? max.doubleValue() : 2.147483647E9D);
                defaultValue = (defaultValue != null) ? defaultValue : min;
                sDRParameterUint8 = new SDRParameterUint8(id.intValue(), name, description, category, defaultValue.intValue(), defaultValue.intValue(), max.intValue(), min.intValue(), step.intValue());
                break;
              case "uint_32":
                min = Double.valueOf((min != null) ? min.doubleValue() : 0.0D);
                max = Double.valueOf((max != null) ? max.doubleValue() : 4.294967295E9D);
                defaultValue = (defaultValue != null) ? defaultValue : min;
                sDRParameterUInt32 = new SDRParameterUInt32(id.intValue(), name, description, category, defaultValue.longValue(), defaultValue.longValue(), max.longValue(), min.longValue(), step.longValue());
                break;
            }
          }
          if (sDRParameterUInt32 != null)
            parameters.add(sDRParameterUInt32);
        } catch (Exception e) {
          e.printStackTrace();
          System.err.println("Error parsing sdr parameter");
        }
      }
    } catch (IOException|org.json.simple.parser.ParseException e) {
      e.printStackTrace();
    }
    return parameters;
  }
}
