package ubiik.wp.ami.model;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiEvent extends AMIMessage {
  public static final int AMI_EVENT_MIN_SIZE = 6;
  
  private Integer amiTime;
  
  private Integer value;
  
  public AmiEvent(JSONObject json) {
    super(json);
    this.amiTime = JsonUtils.getIntIfExists(json, "time");
    this.value = JsonUtils.getIntIfExists(json, "value");
    try {
      ByteBuffer buf = ByteBuffer.allocate(6).order(ByteOrder.LITTLE_ENDIAN);
      buf.putInt(this.amiTime.intValue());
      buf.putShort(this.value.shortValue());
      this.payload.write(buf.array());
    } catch (IOException e) {
      e.printStackTrace();
    } 
  }
  
  public AmiEvent(byte[] bytes) {
    super(bytes);
    if (bytes.length < 8)
      throw new IllegalArgumentException("Data is too small for AmiRawAxdr"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.amiTime = Integer.valueOf(buf.getInt());
    this.value = Integer.valueOf(0xFFFF & buf.getShort());
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiEvent otherMsg = (AmiEvent)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.amiTime, otherMsg.getAmiTime(), success, message, "time didn't match\n");
    success = compareIfNotNull(this.value, otherMsg.getValue(), success, message, "value didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getAmiTime() {
    return this.amiTime;
  }
  
  public Integer getValue() {
    return this.value;
  }
}
