package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiGet extends AMIMessage {
  public static final int AMI_GET_SIZE = 15;
  
  public static final byte AmiSelectNone = 0;
  
  public static final byte AmiSelectByEntry = 1;
  
  public static final byte AmiSelectByDate = 2;
  
  private Integer requestId;
  
  private Integer classId;
  
  private DLMSObis obisCode;
  
  private Integer attributeId;
  
  private Integer selectType;
  
  private Long reqBlock;
  
  private Integer start;
  
  private Integer end;
  
  public AmiGet(byte[] bytes) {
    super(bytes);
    if (bytes.length < 17)
      throw new IllegalArgumentException("Data is too small for AmiGet"); 
    this.start = null;
    this.end = null;
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.requestId = Integer.valueOf(0xFF & buf.get());
    this.classId = Integer.valueOf(0xFFFF & buf.getShort());
    this.obisCode = new DLMSObis(buf);
    this.attributeId = Integer.valueOf(0xFF & buf.get());
    this.selectType = Integer.valueOf(0xFF & buf.get());
    this.reqBlock = Long.valueOf(0xFFFFFFFFL & buf.getInt());
    if (this.selectType.intValue() == 1) {
      this.start = Integer.valueOf(0xFFFF & buf.getShort());
      this.end = Integer.valueOf(0xFFFF & buf.getShort());
    } else if (this.selectType.intValue() == 2) {
      this.start = Integer.valueOf(buf.getInt());
      this.end = Integer.valueOf(buf.getInt());
    } 
  }
  
  public AmiGet(JSONObject json) {
    super(json);
    this.requestId = JsonUtils.getIntIfExists(json, "requestId");
    this.classId = JsonUtils.getIntIfExists(json, "classId");
    this.obisCode = new DLMSObis(json);
    this.attributeId = JsonUtils.getIntIfExists(json, "attributeId");
    this.selectType = JsonUtils.getIntIfExists(json, "selectType");
    this.reqBlock = JsonUtils.getLongIfExists(json, "reqBlock");
    this.start = JsonUtils.getIntIfExists(json, "start");
    this.end = JsonUtils.getIntIfExists(json, "end");
    try {
      int bufSize = 15;
      if (this.start != null && this.end != null && 
        this.selectType != null)
        if (this.selectType.intValue() == 1) {
          bufSize += 4;
        } else if (this.selectType.intValue() == 2) {
          bufSize += 8;
        }  
      ByteBuffer buf = ByteBuffer.allocate(bufSize).order(ByteOrder.LITTLE_ENDIAN);
      buf.put(this.requestId.byteValue());
      buf.putShort(this.classId.shortValue());
      buf.put(this.obisCode.getPayload());
      buf.put(this.attributeId.byteValue());
      buf.put(this.selectType.byteValue());
      buf.putInt(this.reqBlock.intValue());
      if (this.start != null && this.end != null && 
        this.selectType != null)
        if (this.selectType.intValue() == 1) {
          buf.putShort(this.start.shortValue());
          buf.putShort(this.end.shortValue());
        } else if (this.selectType.intValue() == 2) {
          buf.putInt(this.start.intValue());
          buf.putInt(this.end.intValue());
        }  
      this.payload.write(buf.array());
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }
  
  public Integer getRequestId() {
    return this.requestId;
  }
  
  public Integer getClassId() {
    return this.classId;
  }
  
  public DLMSObis getObisCode() {
    return this.obisCode;
  }
  
  public Integer getAttributeId() {
    return this.attributeId;
  }
  
  public Integer getSelectType() {
    return this.selectType;
  }
  
  public Long getReqBlock() {
    return this.reqBlock;
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    return new AmiComparisonResult(false, "Not implemented");
  }
}
