package ubiik.wp.ami.model;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiMeterDiscovery extends AMIMessage {
  public static final int METER_UNIQUE_ID_SIZE = 2;
  
  public static final int METER_ID_SIZE = 8;
  
  public static final int METER_TYPE_SIZE = 3;
  
  public static final int METER_DISCOVERY_SIZE = 14;
  
  private byte amiAssocStatus;
  
  private byte[] meterUniqueId;
  
  private byte[] meterId;
  
  private byte[] meterType;
  
  public AmiMeterDiscovery(byte[] bytes) {
    super(bytes);
    if (bytes.length < 16)
      throw new IllegalArgumentException("Data is too small for AmiMeterDiscovery"); 
    this.meterUniqueId = new byte[2];
    this.meterId = new byte[8];
    this.meterType = new byte[3];
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.amiAssocStatus = buf.get();
    buf.get(this.meterUniqueId, 0, 2);
    buf.get(this.meterId, 0, 8);
    buf.get(this.meterType, 0, 3);
  }
  
  public AmiMeterDiscovery(AMIMessage.Direction dir, byte amiAssocStatus, byte[] meterUniqueId, byte[] meterId, byte[] meterType) {
    super((byte)-96, dir);
    this.amiAssocStatus = amiAssocStatus;
    this.meterUniqueId = meterUniqueId;
    this.meterId = meterId;
    this.meterType = meterType;
    try {
      ByteBuffer buffer = ByteBuffer.allocate(14).order(ByteOrder.LITTLE_ENDIAN);
      buffer.put(this.amiAssocStatus);
      buffer.put(this.meterUniqueId);
      buffer.put(this.meterId);
      buffer.put(this.meterType);
      this.payload.write(buffer.array());
    } catch (IOException e) {
      e.printStackTrace();
    } 
  }
  
  public AmiMeterDiscovery(JSONObject json) {
    super(json);
    this.amiAssocStatus = ((Long)json.get("associationStatus")).byteValue();
    this.meterUniqueId = JsonUtils.getByteArr(json, "meterUniqueId");
    this.meterId = JsonUtils.getByteArr(json, "meterId");
    this.meterType = JsonUtils.getByteArr(json, "meterType");
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiMeterDiscovery discovery = (AmiMeterDiscovery)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    System.out.println("this.amiAssocStatus: " + this.amiAssocStatus + " discovery.getAmiAssocStatus() " + discovery.getAmiAssocStatus());
    if (this.amiAssocStatus == discovery.getAmiAssocStatus()) {
      success = compareIfNotNull(this.meterUniqueId, discovery.getMeterUniqueId(), success, message, "meterUniqueId didn't match\n");
      success = compareIfNotNull(this.meterId, discovery.getMeterId(), success, message, "meterId didn't match\n");
      success = compareIfNotNull(this.meterType, discovery.getMeterType(), success, message, "meterType didn't match\n");
    } else {
      success = Boolean.valueOf(false);
      message.append("Expected Discovery with AssocStatus ")
        .append(this.amiAssocStatus).append(" but received ")
        .append(discovery.getAmiAssocStatus());
    } 
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public byte getAmiAssocStatus() {
    return this.amiAssocStatus;
  }
  
  public byte[] getMeterUniqueId() {
    return this.meterUniqueId;
  }
  
  public byte[] getMeterId() {
    return this.meterId;
  }
  
  public byte[] getMeterType() {
    return this.meterType;
  }
}
