package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiNotification extends AMIMessage {
  public static final int NOTIFICATION_SIZE = 1;
  
  public static final byte AmiNotificationOutage = 0;
  
  private Integer notificationType;
  
  public AmiNotification(JSONObject json) {
    super(json);
    this.notificationType = JsonUtils.getIntIfExists(json, "notification");
    this.payload.write(this.notificationType.byteValue());
  }
  
  public AmiNotification(byte[] bytes) {
    super(bytes);
    if (bytes.length < 3)
      throw new IllegalArgumentException("Data is too small for AmiFanParameters"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.notificationType = Integer.valueOf(0xFF & buf.get());
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiNotification otherMsg = (AmiNotification)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.notificationType, otherMsg.getNotificationType(), success, message, "notificationType didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getNotificationType() {
    return this.notificationType;
  }
}
