package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiRawDlms extends AMIMessage {
  private Integer requestId;
  
  private byte[] axdr;
  
  public static final int MIN_SIZE_AMI_RAW_DLMS = 1;
  
  public AmiRawDlms(byte[] bytes) {
    super(bytes);
    if (bytes.length < 3)
      throw new IllegalArgumentException("Data is too small for AmiRawAxdr"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.requestId = Integer.valueOf(0xFF & buf.get());
    this.axdr = new byte[bytes.length - buf.position()];
    buf.get(this.axdr);
  }
  
  public AmiRawDlms(JSONObject json) {
    super(json);
    this.requestId = JsonUtils.getIntIfExists(json, "requestId");
    this.axdr = JsonUtils.getByteArr(json, "axdr");
    try {
      this.payload.write(this.requestId.byteValue());
      if (this.axdr != null)
        this.payload.write(this.axdr); 
    } catch (Exception e) {
      System.err.println("Message" + getClass().getSimpleName() + "is incomplete to create payload");
    } 
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiRawDlms resp = (AmiRawDlms)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.requestId, resp.getRequestId(), success, message, "requestId didn't match\n");
    success = compareIfNotNull(this.axdr, resp.getAxdr(), success, message, "axdr didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getRequestId() {
    return this.requestId;
  }
  
  public byte[] getAxdr() {
    return this.axdr;
  }
}
