package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiResponse extends AMIMessage {
  public static final int MIN_AMI_GET_RESP_SIZE = 2;
  
  private Integer requestId;
  
  private Integer status;
  
  private byte[] axdr;
  
  public AmiResponse(byte[] bytes) {
    super(bytes);
    if (bytes.length < 4)
      throw new IllegalArgumentException("Data is too small for AmiGetResponse"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.requestId = Integer.valueOf(0xFF & buf.get());
    this.status = Integer.valueOf(0xFF & buf.get());
    this.axdr = new byte[bytes.length - buf.position()];
    buf.get(this.axdr, 0, this.axdr.length);
  }
  
  public AmiResponse(JSONObject json) {
    super(json);
    this.requestId = JsonUtils.getIntIfExists(json, "requestId");
    this.status = JsonUtils.getIntIfExists(json, "status");
    this.axdr = JsonUtils.getByteArr(json, "axdr");
    try {
      ByteBuffer buf = ByteBuffer.allocate(2).order(ByteOrder.LITTLE_ENDIAN);
      buf.put(this.requestId.byteValue());
      buf.put(this.status.byteValue());
      this.payload.write(buf.array());
      if (this.axdr != null)
        this.payload.write(this.axdr); 
    } catch (Exception e) {
      System.err.println("Message AmiGetResponse is incomplete to create payload");
    } 
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    AmiResponse resp = (AmiResponse)other;
    Boolean success = Boolean.valueOf(true);
    StringBuilder message = new StringBuilder();
    success = compareIfNotNull(this.requestId, resp.getRequestId(), success, message, "requestId didn't match\n");
    success = compareIfNotNull(this.status, resp.getStatus(), success, message, "status didn't match\n");
    success = compareIfNotNull(this.axdr, resp.getAxdr(), success, message, "axdr didn't match\n");
    return new AmiComparisonResult(success.booleanValue(), message.toString());
  }
  
  public Integer getRequestId() {
    return this.requestId;
  }
  
  public Integer getStatus() {
    return this.status;
  }
  
  public byte[] getAxdr() {
    return this.axdr;
  }
}
