package ubiik.wp.ami.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import org.json.simple.JSONObject;
import ubiik.utils.JsonUtils;

public class AmiSet extends AMIMessage {
  public static final int MIN_AMI_SET_SIZE = 4;
  
  private Integer requestId;
  
  private Integer classId;
  
  private DLMSObis obisCode;
  
  private Integer attributeId;
  
  private byte[] axdr;
  
  public AmiSet(byte[] bytes) {
    super(bytes);
    if (bytes.length < 6)
      throw new IllegalArgumentException("Data is too small for AmiGet"); 
    ByteBuffer buf = ByteBuffer.wrap(bytes).order(ByteOrder.LITTLE_ENDIAN);
    buf.position(2);
    this.requestId = Integer.valueOf(0xFF & buf.get());
    this.classId = Integer.valueOf(0xFFFF & buf.getShort());
    this.obisCode = new DLMSObis(buf);
    this.attributeId = Integer.valueOf(0xFF & buf.get());
    this.axdr = new byte[bytes.length - buf.position()];
    buf.get(this.axdr);
  }
  
  public AmiSet(JSONObject json) {
    super(json);
    this.requestId = JsonUtils.getIntIfExists(json, "requestId");
    this.classId = JsonUtils.getIntIfExists(json, "classId");
    this.obisCode = new DLMSObis(json);
    this.attributeId = JsonUtils.getIntIfExists(json, "attributeId");
    this.axdr = JsonUtils.getByteArr(json, "axdr");
    try {
      ByteBuffer buf = ByteBuffer.allocate(4 + (this.obisCode.getPayload()).length).order(ByteOrder.LITTLE_ENDIAN);
      buf.put(this.requestId.byteValue());
      buf.putShort(this.classId.shortValue());
      buf.put(this.obisCode.getPayload());
      buf.put(this.attributeId.byteValue());
      this.payload.write(buf.array());
      if (this.axdr != null)
        this.payload.write(this.axdr); 
    } catch (Exception e) {
      e.printStackTrace();
    } 
  }
  
  protected AmiComparisonResult doAmiCompare(AMIMessage other) {
    return new AmiComparisonResult(false, "Not implemented");
  }
}
