package ubiik.wp.controller;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import ubiik.wp.model.messages.PSMessage;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandStringParameter;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseStringParameter;
import ubiik.wp.model.testmode.TestModeFactory;
import ubiik.wp.model.testmode.TestModesListener;
import ubiik.wp.model.testmode.UUTestMode;

public final class TestModesManager implements PSCommandDelegate {
  private List<UUTestMode> testModes = new ArrayList<>();
  
  private List<TestModesListener> listeners = new LinkedList<>();
  
  private MessageController messageController = null;
  
  private static class Holder {
    static final TestModesManager INSTANCE = new TestModesManager();
  }
  
  public static TestModesManager getInstance() {
    return Holder.INSTANCE;
  }
  
  public synchronized List<UUTestMode> getTestModes() {
    return new ArrayList<>(this.testModes);
  }
  
  private synchronized void notifyListeners(boolean success) {
    for (TestModesListener listener : this.listeners) {
      if (success) {
        listener.updateTestModes();
        continue;
      } 
      listener.updateTestModesFailed();
    } 
  }
  
  public synchronized void addListener(TestModesListener listener) {
    this.listeners.add(listener);
  }
  
  public synchronized void removeListener(TestModesListener listener) {
    this.listeners.remove(listener);
  }
  
  public synchronized void refreshTestModes() {
    this.messageController.sendMessage((PSMessage)new PSCommandStringParameter(this, "TEST_MODES", null, (byte)0));
  }
  
  public synchronized void commandandReceivedResponse(PSCommand command) {
    boolean success = false;
    if (command.getResponse().isSuccess() && 
      command instanceof PSCommandStringParameter) {
      PSCommandStringParameter strCmd = (PSCommandStringParameter)command;
      if (strCmd.getParameter().compareTo("TEST_MODES") == 0 && 
        strCmd.getResponse() instanceof PSCommandResponseStringParameter) {
        String respContent = ((PSCommandResponseStringParameter)strCmd.getResponse()).getContent();
        TestModeFactory factory = new TestModeFactory();
        ArrayList<UUTestMode> nTestModes = factory.loadTestModes(respContent);
        this.testModes = nTestModes;
        success = true;
      } 
    } 
    notifyListeners(success);
  }
  
  public synchronized void setMessageController(MessageController mc) {
    this.messageController = mc;
  }
  
  private TestModesManager() {}
}
