package ubiik.wp.demo.controller;

import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.chart.LineChart;
import javafx.scene.chart.XYChart;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.layout.GridPane;
import javafx.util.Callback;
import ubiik.utils.view.ColumnFormatter;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.demo.model.ADCGyroSample;
import ubiik.wp.model.messages.PSUplink;
import ubiik.wp.view.DisplayConstants;

public class ADCGyroViewController extends UplinksDemoViewController {
  private static final String NAME_ADC1 = "ADC1";
  
  private static final String NAME_ADC2 = "ADC2";
  
  private static final String NAME_ADC3 = "ADC3";
  
  private static final String NAME_ADC4 = "ADC4";
  
  private static final String NAME_GYRO_X = "X";
  
  private static final String NAME_GYRO_Y = "Y";
  
  private static final String NAME_GYRO_Z = "Z";
  
  private LineChart<Date, Number> ADC1Chart;
  
  private LineChart<Date, Number> ADC2_3_Chart;
  
  private LineChart<Date, Number> ADC4Chart;
  
  private LineChart<Date, Number> XYZChart;
  
  @FXML
  private GridPane gridPane;
  
  @FXML
  private ChoiceBox<String> choiceBoxEndDevices;
  
  private List<String> availableEds;
  
  private String currentEd;
  
  private Map<String, Map<String, ObservableList<XYChart.Data<Date, Number>>>> dataForEd;
  
  @FXML
  protected TableView<ADCGyroSample> tableView;
  
  private ObservableList<ADCGyroSample> latestSamples;
  
  public ADCGyroViewController(DevicesController devicesController, MessageController messageController) {
    super(devicesController, messageController);
    this.dataForEd = new HashMap<>();
    this.availableEds = new LinkedList<>();
    this.latestSamples = FXCollections.observableArrayList();
  }
  
  public void initialize() {
    super.initialize();
    this.choiceBoxEndDevices.valueProperty().addListener(new ChangeListener<String>() {
          public void changed(ObservableValue ov, String t, String t1) {
            ADCGyroViewController.this.choiceboxEdChanged();
          }
        });
    this.chartsContainer = this.gridPane;
    setUpCharts();
    setTitle("ADC and Gyro Demo");
    setUpSamplesTable();
  }
  
  private void setUpCharts() {
    this.ADC1Chart = setUpLineChart(this.ADC1Chart, FXCollections.observableArrayList(), "Water Sensor (ADC1)");
    this.ADC2_3_Chart = setUpLineChart(this.ADC2_3_Chart, FXCollections.observableArrayList(), "Tilt Meter (ADC2 / ADC3)");
    this.ADC4Chart = setUpLineChart(this.ADC4Chart, FXCollections.observableArrayList(), "Crack Meter (ADC4)");
    this.XYZChart = setUpLineChart(this.XYZChart, FXCollections.observableArrayList(), "Gyro (XYZ)");
    this.gridPane.add(this.ADC1Chart, 0, 0);
    this.gridPane.add(this.ADC2_3_Chart, 0, 1);
    this.gridPane.add(this.ADC4Chart, 1, 0);
    this.gridPane.add(this.XYZChart, 1, 1);
  }
  
  private void setUpSamplesTable() {
    this.tableView.getColumns().clear();
    TableColumn<ADCGyroSample, String> deviceCol = new TableColumn<>("End Device");
    TableColumn<ADCGyroSample, Double> adc1Col = createDoublePropertyCol("ADC1 (mV)", "ADC1");
    TableColumn<ADCGyroSample, Double> adc2Col = createDoublePropertyCol("ADC2 (mV)", "ADC2");
    TableColumn<ADCGyroSample, Double> adc3Col = createDoublePropertyCol("ADC3 (mV)", "ADC3");
    TableColumn<ADCGyroSample, Double> adc4Col = createDoublePropertyCol("ADC4 (mV)", "ADC4");
    TableColumn<ADCGyroSample, Double> gyroXCol = createDoublePropertyCol("Gyro x", "gyroX");
    TableColumn<ADCGyroSample, Double> gyroYCol = createDoublePropertyCol("Gyro y", "gyroY");
    TableColumn<ADCGyroSample, Double> gyroZCol = createDoublePropertyCol("Gyro z", "gyroZ");
    TableColumn<ADCGyroSample, Date> datelCol = new TableColumn<>("Date");
    deviceCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.25D));
    datelCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.12D));
    deviceCol.setSortable(true);
    datelCol.setSortable(true);
    deviceCol.setCellValueFactory(new PropertyValueFactory<>("uuid"));
    datelCol.setCellValueFactory(new PropertyValueFactory<>("timestamp"));
    datelCol.setCellFactory((Callback<TableColumn<ADCGyroSample, Date>, TableCell<ADCGyroSample, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    this.tableView.getColumns().addAll(new TableColumn[] { deviceCol, datelCol, adc1Col, adc2Col, adc3Col, adc4Col, gyroXCol, gyroYCol, gyroZCol });
    this.tableView.setItems(this.latestSamples);
  }
  
  private TableColumn<ADCGyroSample, Double> createDoublePropertyCol(String title, String propertyName) {
    TableColumn<ADCGyroSample, Double> propertyCol = new TableColumn<>(title);
    propertyCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.09D));
    propertyCol.setSortable(true);
    propertyCol.setCellValueFactory(new PropertyValueFactory<>(propertyName));
    return propertyCol;
  }
  
  protected void processUplink(final PSUplink uplink) {
    ADCGyroSample sample = ADCGyroSample.getSampleFromUplink(uplink);
    if (sample != null) {
      addSample(sample);
      if (this.currentEd == null) {
        setCurrenEndDevice(uplink.getEndDevice().getUUIDString());
        Platform.runLater(new Runnable() {
              public void run() {
                ADCGyroViewController.this.choiceBoxEndDevices.getSelectionModel().select(uplink.getEndDevice().getUUIDString());
              }
            });
      } 
      updateLatestSamples(sample);
    } 
  }
  
  private Map<String, ObservableList<XYChart.Data<Date, Number>>> getOrCreateListsForEd(String uueid) {
    Map<String, ObservableList<XYChart.Data<Date, Number>>> listsForDevice = this.dataForEd.get(uueid);
    if (listsForDevice == null) {
      listsForDevice = new HashMap<>();
      listsForDevice.put("ADC1", FXCollections.observableArrayList());
      listsForDevice.put("ADC2", FXCollections.observableArrayList());
      listsForDevice.put("ADC3", FXCollections.observableArrayList());
      listsForDevice.put("ADC4", FXCollections.observableArrayList());
      listsForDevice.put("X", FXCollections.observableArrayList());
      listsForDevice.put("Y", FXCollections.observableArrayList());
      listsForDevice.put("Z", FXCollections.observableArrayList());
      this.dataForEd.put(uueid, listsForDevice);
    } 
    return listsForDevice;
  }
  
  private synchronized void addSample(ADCGyroSample sample) {
    Map<String, ObservableList<XYChart.Data<Date, Number>>> listsForDevice = getOrCreateListsForEd(sample.getUuid());
    Date sampleTime = sample.getTimestamp();
    ((ObservableList)listsForDevice.get("ADC1")).add(new XYChart.Data<>(sampleTime, Double.valueOf(sample.getADC1())));
    ((ObservableList)listsForDevice.get("ADC2")).add(new XYChart.Data<>(sampleTime, Double.valueOf(sample.getADC2())));
    ((ObservableList)listsForDevice.get("ADC3")).add(new XYChart.Data<>(sampleTime, Double.valueOf(sample.getADC3())));
    ((ObservableList)listsForDevice.get("ADC4")).add(new XYChart.Data<>(sampleTime, Double.valueOf(sample.getADC4())));
    ((ObservableList)listsForDevice.get("X")).add(new XYChart.Data<>(sampleTime, Double.valueOf(sample.getGyroX())));
    ((ObservableList)listsForDevice.get("Y")).add(new XYChart.Data<>(sampleTime, Double.valueOf(sample.getGyroY())));
    ((ObservableList)listsForDevice.get("Z")).add(new XYChart.Data<>(sampleTime, Double.valueOf(sample.getGyroZ())));
    updateEndDevices(sample);
    System.out.print("Sample: " + sample.getUuid() + " ");
    System.out.print("ADC1: " + sample.getADC1() + " ");
    System.out.print("ADC2: " + sample.getADC2() + " ");
    System.out.print("ADC3: " + sample.getADC3() + " ");
    System.out.print("ADC4: " + sample.getADC4() + " ");
    System.out.print("X: " + sample.getGyroX() + " ");
    System.out.print("Y: " + sample.getGyroY() + " ");
    System.out.print("Z: " + sample.getGyroZ() + " ");
    System.out.println();
  }
  
  private synchronized void updateEndDevices(final ADCGyroSample sample) {
    if (!this.availableEds.contains(sample.getUuid())) {
      this.availableEds.add(sample.getUuid());
      Platform.runLater(new Runnable() {
            public void run() {
              ADCGyroViewController.this.choiceBoxEndDevices.getItems().add(sample.getUuid());
            }
          });
    } 
  }
  
  public void choiceboxEdChanged() {
    setCurrenEndDevice(this.choiceBoxEndDevices.getSelectionModel().getSelectedItem());
  }
  
  private synchronized void setCurrenEndDevice(String endDevice) {
    Map<String, ObservableList<XYChart.Data<Date, Number>>> listsForDevice = getOrCreateListsForEd(endDevice);
    ObservableList<XYChart.Series<Date, Number>> adc1Series = FXCollections.observableArrayList();
    adc1Series.add(new XYChart.Series<>("ADC1", listsForDevice.get("ADC1")));
    this.ADC1Chart.setData(adc1Series);
    ObservableList<XYChart.Series<Date, Number>> adc23Series = FXCollections.observableArrayList();
    adc23Series.add(new XYChart.Series<>("ADC2", listsForDevice.get("ADC2")));
    adc23Series.add(new XYChart.Series<>("ADC3", listsForDevice.get("ADC3")));
    this.ADC2_3_Chart.setData(adc23Series);
    ObservableList<XYChart.Series<Date, Number>> adc4Series = FXCollections.observableArrayList();
    adc4Series.add(new XYChart.Series<>("ADC4", listsForDevice.get("ADC4")));
    this.ADC4Chart.setData(adc4Series);
    ObservableList<XYChart.Series<Date, Number>> xyzSeries = FXCollections.observableArrayList();
    xyzSeries.add(new XYChart.Series<>("X", listsForDevice.get("X")));
    xyzSeries.add(new XYChart.Series<>("Y", listsForDevice.get("Y")));
    xyzSeries.add(new XYChart.Series<>("Z", listsForDevice.get("Z")));
    this.XYZChart.setData(xyzSeries);
    this.currentEd = endDevice;
  }
  
  protected LineChart<Date, Number> setUpLineChart(LineChart<Date, Number> lineChart, ObservableList<XYChart.Series<Date, Number>> series, String title) {
    if (lineChart != null)
      this.chartsContainer.getChildren().remove(lineChart); 
    lineChart = createDateAxisChart(series);
    lineChart.setAnimated(true);
    lineChart.setTitle(title);
    return lineChart;
  }
  
  private void updateLatestSamples(ADCGyroSample sample) {
    String sampleUuid = sample.getUuid();
    Optional<ADCGyroSample> matchingObjects = this.latestSamples.stream().filter(p -> p.getUuid().equals(sampleUuid)).findAny();
    ADCGyroSample oldSample = matchingObjects.orElse(null);
    if (oldSample != null)
      this.latestSamples.remove(oldSample); 
    this.latestSamples.add(sample);
  }
  
  public boolean getBSInfo() {return false;}
}
