package ubiik.wp.demo.controller;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Date;
import java.util.Optional;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import org.controlsfx.control.table.TableFilter;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.demo.model.PhySample;
import ubiik.wp.model.messages.PSUplink;
import ubiik.wp.util.ApplicationLogger;

public class ListOnlyRSSI extends UplinksDemoViewController {
  private ObservableList<PhySample> latestPhySamples = FXCollections.observableArrayList();
  
  @FXML
  protected TableView<PhySample> tableView;
  
  private TableFilter<PhySample> filter;
  
  public ListOnlyRSSI(DevicesController devicesController, MessageController messageController) {
    super(devicesController, messageController);
  }
  
  public void authenticationFinished(boolean autheticated) {
    super.authenticationFinished(autheticated);
  }
  
  public void initialize() {
    super.initialize();
    setUpSamplesTable();
    setTitle("Phy Statistics");
    this.filter = TableFilter.forTableView(this.tableView).apply();
  }
  
  private void setUpSamplesTable() {
    this.tableView.getColumns().clear();
    TableColumn<PhySample, String> deviceCol = new TableColumn<>("End Device");
    TableColumn<PhySample, Integer> appRssiCol = new TableColumn<>("Downlink RSSI");
    TableColumn<PhySample, Integer> uplinkRssiCol = new TableColumn<>("Uplink RSSI");
    TableColumn<PhySample, String> datelCol = new TableColumn<>("Date");
    deviceCol.prefWidthProperty().set(350.0D);
    appRssiCol.prefWidthProperty().set(200.0D);
    uplinkRssiCol.prefWidthProperty().set(200.0D);
    datelCol.prefWidthProperty().set(250.0D);
    deviceCol.setSortable(true);
    appRssiCol.setSortable(false);
    uplinkRssiCol.setSortable(false);
    datelCol.setSortable(false);
    deviceCol.setCellValueFactory(new PropertyValueFactory<>("uuid"));
    appRssiCol.setCellValueFactory(new PropertyValueFactory<>("rssi"));
    uplinkRssiCol.setCellValueFactory(new PropertyValueFactory<>("uplinkRssi"));
    datelCol.setCellValueFactory(new PropertyValueFactory<>("uplinkTimeStr"));
    this.tableView.getColumns().addAll(new TableColumn[] { deviceCol, appRssiCol, uplinkRssiCol, datelCol });
    this.tableView.setItems(this.latestPhySamples);
  }
  
  private PhySample getPhyReadingFromUplink(PSUplink uplink) {
    PhySample phySample = null;
    byte[] data = uplink.getApplicationData();
    try {
      if (data.length == 13) {
        ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.LITTLE_ENDIAN);
        byte appType = buffer.get();
        if (appType == 32) {
          int npkt = 0xFFFF & buffer.getShort();
          int rssi = buffer.getShort();
          int batteryVoltage = buffer.getShort();
          int temperature = buffer.getShort();
          long intSampleTime = buffer.getInt();
          Date sampleTime = new Date(intSampleTime * 1000L);
          ApplicationLogger.debug("Sample time: " + intSampleTime + " " + sampleTime.toString());
          phySample = new PhySample(uplink.getTargetString(), uplink.getTimestamp(), uplink.getRssi(), npkt, rssi, batteryVoltage, temperature, uplink.getCreationDate(), sampleTime);
          ApplicationLogger.debug("New sample: " + phySample.toString());
        } 
      } 
    } catch (Exception e) {
      e.printStackTrace();
    } 
    return phySample;
  }
  
  protected void processUplink(PSUplink uplink) {
    final PhySample sample = getPhyReadingFromUplink(uplink);
    if (sample != null)
      Platform.runLater(new Runnable() {
            public void run() {
              ListOnlyRSSI.this.updateLatestSamples(sample);
            }
          }); 
  }
  
  private void updateLatestSamples(PhySample sample) {
    String sampleUuid = sample.getUuid();
    Optional<PhySample> matchingObjects = this.latestPhySamples.stream().filter(p -> p.getUuid().equals(sampleUuid)).findAny();
    PhySample sampleInTable = matchingObjects.orElse(null);
    if (sampleInTable == null) {
      sampleInTable = new PhySample(sample.getUuid(), sample.getTimestamp(), sample.getUplinkRssi(), sample.getPackets(), sample.getRssi(), sample.getBatteryVoltage(), sample.getTemperature(), sample.getUplinkTime(), sample.getSampleTime());
      this.latestPhySamples.add(sampleInTable);
    } else {
      sampleInTable.setTimestamp(sample.getTimestamp());
      sampleInTable.setUplinkRssi(sample.getUplinkRssi());
      sampleInTable.setPackets(sample.getPackets());
      sampleInTable.setRssi(sample.getRssi());
      sampleInTable.setBatteryVoltage(sample.getBatteryVoltage());
      sampleInTable.setTemperature(sample.getTemperature());
      sampleInTable.setUplinkTime(sample.getUplinkTime());
      sampleInTable.setSampleTime(sample.getSampleTime());
    } 
    this.tableView.refresh();
  }
  
  public boolean getBSInfo() {return false;}
}
