package ubiik.wp.demo.model;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Date;
import ubiik.wp.model.messages.PSUplink;
import ubiik.wp.util.ApplicationLogger;

public class ADCGyroSample extends SampleFromUplink {
  private static final double ADC1_MULT_CONST = 20.0D;
  
  private static final double ADC1_DIV_CONST = 65535.0D;
  
  private static final double ADC_2_3_MAX = 9.958D;
  
  private static final double ADC_2_3_SCALE_POS = 32767.0D;
  
  private static final double ADC_2_3_SCALE_NEG = 32768.0D;
  
  private double ADC1;
  
  private double ADC2;
  
  private double ADC3;
  
  private double ADC4;
  
  private double gyroX;
  
  private double gyroY;
  
  private double gyroZ;
  
  public ADCGyroSample(Date timestamp, String uuid, double aDC1, double aDC2, double aDC3, double aDC4, double gyroX, double gyroY, double gyroZ) {
    super(timestamp, uuid);
    this.ADC1 = aDC1;
    this.ADC2 = aDC2;
    this.ADC3 = aDC3;
    this.ADC4 = aDC4;
    this.gyroX = gyroX;
    this.gyroY = gyroY;
    this.gyroZ = gyroZ;
  }
  
  public static ADCGyroSample getSampleFromUplink(PSUplink uplink) {
    ADCGyroSample adcGyroSample = null;
    byte[] data = uplink.getApplicationData();
    try {
      ByteBuffer buffer = ByteBuffer.wrap(data).order(ByteOrder.BIG_ENDIAN);
      if (buffer.get() == 19 && buffer
        .get() == 0) {
        int raw_adc1 = 0xFFFF & buffer.getShort();
        short raw_adc2 = buffer.getShort();
        short raw_adc3 = buffer.getShort();
        int raw_adc4 = 0xFFFF & buffer.getShort();
        short x = buffer.getShort();
        short y = buffer.getShort();
        double z = buffer.getShort();
        double adc1 = raw_adc1 * 20.0D / 65535.0D;
        double adc4 = raw_adc4 * 20.0D / 65535.0D;
        double adc2 = getADC2_3_value(raw_adc2);
        double adc3 = getADC2_3_value(raw_adc3);
        adcGyroSample = new ADCGyroSample(uplink.getTimestamp(), uplink.getTargetString(), adc1, adc2, adc3, adc4, x, y, z);
        ApplicationLogger.debug("New sample: " + adcGyroSample.toString());
      } 
    } catch (Exception exception) {}
    return adcGyroSample;
  }
  
  private static final double getADC2_3_value(double rawValue) {
    if (rawValue >= 0.0D)
      return rawValue * 9.958D / 32767.0D; 
    return rawValue * 9.958D / 32768.0D;
  }
  
  public double getADC1() {
    return this.ADC1;
  }
  
  public void setADC1(double aDC1) {
    this.ADC1 = aDC1;
  }
  
  public double getADC2() {
    return this.ADC2;
  }
  
  public void setADC2(double aDC2) {
    this.ADC2 = aDC2;
  }
  
  public double getADC3() {
    return this.ADC3;
  }
  
  public void setADC3(double aDC3) {
    this.ADC3 = aDC3;
  }
  
  public double getADC4() {
    return this.ADC4;
  }
  
  public void setADC4(double aDC4) {
    this.ADC4 = aDC4;
  }
  
  public double getGyroX() {
    return this.gyroX;
  }
  
  public void setGyroX(double gyroX) {
    this.gyroX = gyroX;
  }
  
  public double getGyroY() {
    return this.gyroY;
  }
  
  public void setGyroY(double gyroY) {
    this.gyroY = gyroY;
  }
  
  public double getGyroZ() {
    return this.gyroZ;
  }
  
  public void setGyroZ(double gyroZ) {
    this.gyroZ = gyroZ;
  }
}
