package ubiik.wp.model.bsparameters;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class BSParameter {
  public static final String CATEGORY_BS = "BS";
  
  public static final String CATEGORY_UT = "UT";
  
  private Long value;
  
  private Long defaultValue;
  
  private long min;
  
  private long max;
  
  private String name;
  
  private String category;
  
  private String uiName;
  
  private String description;
  
  private int index;
  
  public BSParameter(Long value, Long defaultValue, long min, long max, String name, String category, String uiName, String description, int index) {
    this.value = value;
    this.defaultValue = defaultValue;
    this.min = min;
    this.max = max;
    this.name = name;
    this.uiName = uiName;
    this.description = description;
    this.index = index;
    this.category = category;
  }
  
  public BSParameter(BSParameter param) {
    this.value = (param.getValue() != null) ? new Long(param.getValue().longValue()) : null;
    this.min = param.getMin();
    this.max = param.getMax();
    this.name = new String(param.getName());
    this.uiName = new String(param.getUiName());
    this.description = new String(param.getDescription());
    this.index = param.getIndex();
    this.category = new String(param.getCategory());
  }
  
  public synchronized boolean validateValue(String val) {
    boolean valid = true;
    try {
      long longVal = Long.parseLong(val);
      if (longVal > this.max || longVal < this.min)
        valid = false; 
    } catch (Exception e) {
      valid = false;
    } 
    return valid;
  }
  
  public synchronized boolean validateValue(Long longVal) {
    boolean valid = true;
    if (longVal.longValue() > this.max || longVal.longValue() < this.min)
      valid = false; 
    return valid;
  }
  
  public synchronized boolean setValue(Long value) {
    boolean valid = true;
    if (value != null)
      valid = validateValue(value); 
    if (valid)
      this.value = value; 
    return valid;
  }
  
  public synchronized byte[] getValueInBytes() {
    if (this.value != null)
      return ByteBuffer.allocate(4)
        .order(ByteOrder.LITTLE_ENDIAN)
        .putInt(this.value.intValue())
        .array(); 
    return null;
  }
  
  public String getMinStr() {
    return Long.toString(this.min);
  }
  
  public String getMaxStr() {
    return Long.toString(this.max);
  }
  
  public Long getValue() {
    return this.value;
  }
  
  public String getName() {
    return this.name;
  }
  
  public String getDescription() {
    return this.description;
  }
  
  public String getUiName() {
    return this.uiName;
  }
  
  public int getIndex() {
    return this.index;
  }
  
  public Long getDefaultValue() {
    return this.defaultValue;
  }
  
  public String getValueString() {
    if (this.value != null)
      return this.value.toString(); 
    return "-";
  }
  
  public synchronized long getMin() {
    return this.min;
  }
  
  public synchronized long getMax() {
    return this.max;
  }
  
  public String getCategory() {
    return this.category;
  }
  
  public void setCategory(String category) {
    this.category = category;
  }
}
