package ubiik.wp.model.fpga;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class FPGARegister {
  public static final int BYTES_FOR_FPGA_REGISTER = 4;
  
  private ArrayList<FPGARegisterField> fields;
  
  private int index;
  
  private List<FPGARegisterObserver> observers;
  
  public FPGARegister(int index) {
    this.index = index;
    this.fields = new ArrayList<>();
    this.observers = new LinkedList<>();
  }
  
  public int getAs32Bit() {
    int val = 0;
    for (FPGARegisterField fpgaRegisterField : this.fields)
      val |= fpgaRegisterField.getAs32Bit(); 
    return val;
  }
  
  public void setFrom32Bit(int value) {
    for (FPGARegisterField fpgaRegisterField : this.fields)
      fpgaRegisterField.setFrom32Bit(value); 
    registerChanged();
  }
  
  public int getIndex() {
    return this.index;
  }
  
  public void addField(FPGARegisterField field) {
    this.fields.add(field);
  }
  
  public ArrayList<FPGARegisterField> getFields() {
    return this.fields;
  }
  
  public void registerChanged() {
    synchronized (this.observers) {
      for (FPGARegisterObserver observer : this.observers)
        observer.registerChanged(); 
    } 
  }
  
  public void addObserver(FPGARegisterObserver observer) {
    synchronized (this.observers) {
      this.observers.add(observer);
    } 
  }
  
  public void removeObserver(FPGARegisterObserver observer) {
    synchronized (this.observers) {
      this.observers.remove(observer);
    } 
  }
}
