package ubiik.wp.model.fpga;

public class FPGARegisterField {
  private static final int BITS_FOR_REGISTER = 32;
  
  private static final long MASK_LONG_TO_UNSIGNED_INT = 4294967295L;
  
  private FPGARegister register;
  
  private String name;
  
  private int msb;
  
  private int lsb;
  
  private int value;
  
  private int defaultValue;
  
  private String description;
  
  private boolean writable;
  
  public FPGARegisterField(FPGARegister register, String name, short msb, short lsb, int value, int defaultValue, String description, boolean writable) {
    if (msb < 0 || msb >= 32 || lsb < 0 || lsb > msb || lsb >= 32)
      throw new IllegalArgumentException(); 
    this.register = register;
    this.name = name;
    this.msb = msb;
    this.lsb = lsb;
    this.value = value;
    this.defaultValue = defaultValue;
    this.description = description;
    this.writable = writable;
  }
  
  public int getAs32Bit() {
    int val = this.value << 32 - this.msb - this.lsb + 1;
    val >>>= 32 - this.msb + 1;
    return val;
  }
  
  private int getValueFrom32Bit(int value) {
    int val = value << 32 - this.msb + 1;
    val >>>= 32 - this.msb - this.lsb + 1;
    return val;
  }
  
  public void setFrom32Bit(int value) {
    this.value = getValueFrom32Bit(value);
  }
  
  public boolean validateValue(int value) {
    int exp = this.msb - this.lsb + 1;
    long longValue = 0xFFFFFFFFL & value;
    return (longValue < Math.pow(2.0D, exp));
  }
  
  public String getName() {
    return this.name;
  }
  
  public void setName(String name) {
    this.name = name;
  }
  
  public int getMsb() {
    return this.msb;
  }
  
  public void setMsb(short msb) {
    this.msb = msb;
  }
  
  public int getLsb() {
    return this.lsb;
  }
  
  public void setLsb(short lsb) {
    this.lsb = lsb;
  }
  
  public int getValue() {
    return this.value;
  }
  
  public void setValue(int value) {
    this.value = value;
    this.register.registerChanged();
  }
  
  public int getDefaultValue() {
    return this.defaultValue;
  }
  
  public void setDefaultValue(int defaultValue) {
    this.defaultValue = defaultValue;
  }
  
  public String getDescription() {
    return this.description;
  }
  
  public void setDescription(String description) {
    this.description = description;
  }
  
  public boolean isWritable() {
    return this.writable;
  }
  
  public void setWritable(boolean writable) {
    this.writable = writable;
  }
}
