package ubiik.wp.model.messages;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.HashMap;
import java.util.Map;

public class DownlinkAcknowledgement extends PSMessage {
  private static final int BYTES_FOR_DL_ACK = 8;
  
  public static final int UL_ACK_STS_OK = 0;
  
  public static final int UL_ACK_STS_FRAGMENT_NACK = 64;
  
  public static final int UL_ACK_STS_NACK = 128;
  
  public static final int UL_ACK_STS_BUF_FULL = 129;
  
  public static final int UL_ACK_STS_EID_NOT_FOUND = 130;
  
  public static final int UL_ACK_STS_MTU_EXCEED = 131;
  
  private static final Map<Integer, String> STATUS_NAMES = new HashMap<>();
  
  private int status;
  
  static {
    STATUS_NAMES.put(Integer.valueOf(0), "OK");
    STATUS_NAMES.put(Integer.valueOf(64), "FRAGMENT NAK");
    STATUS_NAMES.put(Integer.valueOf(128), "DL NAK");
    STATUS_NAMES.put(Integer.valueOf(129), "BUFFER FULL");
    STATUS_NAMES.put(Integer.valueOf(130), "NOT FOUND");
    STATUS_NAMES.put(Integer.valueOf(131), "MTU LIMIT");
  }
  
  public DownlinkAcknowledgement(int messageId, byte[] messageData) {
    super(messageId, 0, messageData, (byte)8);
    try {
      ByteBuffer buffer = ByteBuffer.wrap(messageData).order(ByteOrder.LITTLE_ENDIAN);
      this.applicationId = 0xFFFFFFFFL & buffer.getInt();
      this.status = buffer.getInt();
    } catch (Exception e) {
      throw new IllegalArgumentException();
    } 
  }
  
  protected byte[] getMessageDataBytes() throws MessageFormatException {
    ByteBuffer buffer = ByteBuffer.allocate(8).order(ByteOrder.LITTLE_ENDIAN);
    buffer.putInt((int)this.applicationId);
    buffer.putInt(this.status);
    return buffer.array();
  }
  
  public long getDownlinkSequenceNumber() {
    return this.applicationId;
  }
  
  public int getStatus() {
    return this.status;
  }
  
  public void setStatus(int status) {
    this.status = status;
  }
  
  public static String getNameForStatus(Integer status) {
    String name = null;
    if (status != null)
      name = STATUS_NAMES.get(status); 
    if (name == null)
      name = ""; 
    return name;
  }
}
