package ubiik.wp.model.messages;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Date;
import java.util.Arrays;

public class PSMessageDeviceStatus extends PSMessage {
  private static final int BYTES_FOR_ED_TOTAL_UL = 16;
  
  public static final byte DEVICE_TYPE_ED = 0;
  
  public static final byte DEVICE_TYPE_BASE_STATION = 1;
  
  public static final byte DEVICE_TYPE_FOREIGN_ED = 100;
  
  public static final byte DEVICE_TYPE_FOREIGN_BASE_STATION = 101;
  
  public static final byte DEVICE_TYPE_FOREIGN_BASE_STATION_EX = 102;
  
  public static final int BYTES_FOR_BS_VERSION = 4;
  
  public static final int BYTES_FOR_BS_BUILD_TIME = 16;
  
  private byte[] deviceId;
  
  private byte status;
  
  private byte deviceType;
  
  private Date joinTime;
  
  private short rssi;
  
  private Date lastActivityTime;
  
  private Date connectionTime;
  
  private Long totalULACK;
  
  private Long totalULUNACK;
  
  private Byte txMaxPower;
  
  private Byte txPower;
  
  private Integer mtu;
  
  private Integer numEndDevices;
  
  private byte[] firmwareVersion;
  
  private String bsBuildDateTime;
  
  private long flags;
  
  private Long joinFoi;
  
  public PSMessageDeviceStatus(int messageId, byte[] messageData) {
    super(messageId, -1, messageData, (byte)5);
    this.rssi = Short.MAX_VALUE;
    this.joinTime = null;
    this.totalULACK = null;
    this.totalULUNACK = null;
    this.lastActivityTime = null;
    this.connectionTime = null;
    this.firmwareVersion = null;
    this.bsBuildDateTime = null;
    this.numEndDevices = -1;
    try {
      ByteBuffer buffer = ByteBuffer.wrap(messageData).order(ByteOrder.LITTLE_ENDIAN);
      this.deviceType = buffer.get();
      int deviceTypeLen = 0;
      if (this.deviceType == DEVICE_TYPE_ED || this.deviceType == DEVICE_TYPE_FOREIGN_ED) {
        deviceTypeLen = 16;
      } else if (this.deviceType == DEVICE_TYPE_BASE_STATION || this.deviceType == DEVICE_TYPE_FOREIGN_BASE_STATION) {
        deviceTypeLen = 2;
      } else if (this.deviceType == DEVICE_TYPE_FOREIGN_BASE_STATION_EX) {
    	  // ForeignBaseStationEx = 102
    	  
      } 
      else {
    	  System.out.format("this.deviceType = 0x%02x\n", this.deviceType);
        throw new IllegalArgumentException();
      } 

      if (this.deviceType == DEVICE_TYPE_ED || this.deviceType == DEVICE_TYPE_FOREIGN_ED) {
        this.deviceId = new byte[deviceTypeLen];
        buffer.get(this.deviceId);
        this.status = buffer.get();
        
        if (this.deviceType == DEVICE_TYPE_ED) {
          long jtime = 1000L * (buffer.getInt() & 0xFFFFFFFFL);
          this.joinTime = new Date(jtime);
          this.rssi = buffer.getShort();
        } 
        
        long connAT = 1000L * (buffer.getInt() & 0xFFFFFFFFL);
        this.connectionTime = new Date(connAT);

        if (this.deviceType == DEVICE_TYPE_ED) {
          long lastAT = 1000L * (buffer.getInt() & 0xFFFFFFFFL);
          this.lastActivityTime = new Date(lastAT);
          this.totalULUNACK = Long.valueOf(buffer.getInt() & 0xFFFFFFFFL);
          this.totalULACK = Long.valueOf(buffer.getInt() & 0xFFFFFFFFL);
          this.txMaxPower = Byte.valueOf(buffer.get());
          this.txPower = Byte.valueOf(buffer.get());
          this.mtu = Integer.valueOf(buffer.getShort() & 0xFFFF);
        }

        this.firmwareVersion = new byte[4];
        buffer.get(this.firmwareVersion);
        if (this.deviceType == DEVICE_TYPE_ED) {
        	this.flags = Long.valueOf(buffer.getInt() & 0xFFFFFFFFL);
        	this.joinFoi = Long.valueOf(buffer.getInt());
        }
      } else if ((this.deviceType == DEVICE_TYPE_BASE_STATION || this.deviceType == DEVICE_TYPE_FOREIGN_BASE_STATION) && 
        buffer.position() + 20 <= buffer.capacity()) {
        this.deviceId = new byte[deviceTypeLen];
        buffer.get(this.deviceId);
        this.status = buffer.get();
        this.firmwareVersion = new byte[4];
        buffer.get(this.firmwareVersion);
        byte[] charBsBuildDateTime = new byte[16];
        buffer.get(charBsBuildDateTime);
        this.bsBuildDateTime = new String(charBsBuildDateTime);
        this.numEndDevices = buffer.getInt() & 0xffffffff;

      } else if (this.deviceType == DEVICE_TYPE_FOREIGN_BASE_STATION_EX) {
        this.firmwareVersion = new byte[4];
        buffer.get(this.firmwareVersion);
        byte[] charBsBuildDateTime = new byte[16];
        buffer.get(charBsBuildDateTime);
        this.bsBuildDateTime = new String(charBsBuildDateTime);
        this.numEndDevices = buffer.getInt() & 0xffffffff;        
      }
    } catch (Exception e) {
      e.printStackTrace();
      throw new IllegalArgumentException();
    } 
  }
  
  protected byte[] getMessageDataBytes() throws MessageFormatException {
    ByteBuffer buffer = ByteBuffer.allocate(2 + this.deviceId.length).put(this.deviceType).put(this.deviceId).put(this.status);
    return buffer.array();
  }
  
  public boolean isForEndDevice() {
    return (this.deviceType == DEVICE_TYPE_ED || this.deviceType == DEVICE_TYPE_FOREIGN_ED);
  }
  
  public byte[] getDeviceId() {
    return this.deviceId;
  }
  
  public byte getStatus() {
    return this.status;
  }
  
  public byte getDeviceType() {
    return this.deviceType;
  }
  
  public Date getJoinTime() {
    return this.joinTime;
  }
  
  public short getRssi() {
    return this.rssi;
  }
  
  public Date getConnectionTime() {
    return this.connectionTime;
  }
  
  public Date getLastActivityTime() {
    return this.lastActivityTime;
  }
  
  public synchronized Long getTotalULACK() {
    return this.totalULACK;
  }
  
  public synchronized Long getTotalULUNACK() {
    return this.totalULUNACK;
  }
  
  public synchronized byte[] getFirmwareVersion() {
    return this.firmwareVersion;
  }
  
  public synchronized String getBsBuildDateTime() {
    return this.bsBuildDateTime;
  }
  
  public synchronized int getNumEndDevices() {
	  return this.numEndDevices;
  }
  
  public synchronized Long getJoinFoi() {
	  return this.joinFoi;
  }
}
