package ubiik.wp.model.messages;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Date;
import ubiik.wp.model.messages.pscommands.PSCommandPingEndDevice;

public class PSPong extends PSMessage {
  private Date pingTimestamp;
  
  private int dataSize;
  
  private int roundtripDelay;
  
  private int pingId;
  
  private byte[] uueid;
  
  private PSCommandPingEndDevice ping;
  
  public PSPong(int messageId, byte[] messageData) {
    super(messageId, 0, messageData, (byte)13);
    this.ping = null;
    try {
      ByteBuffer buffer = ByteBuffer.wrap(this.data).order(ByteOrder.LITTLE_ENDIAN);
      int headerSize = buffer.getInt();
      int payloadSize = buffer.getInt();
      if (buffer.remaining() != headerSize + payloadSize)
        throw new IllegalArgumentException("Invalid header and payload size"); 
      this.uueid = new byte[16];
      this.protocolVersion = buffer.getInt();
      buffer.get(this.uueid);
      this.pingId = buffer.getInt();
      buffer.position(headerSize + 8);
      this.pingTimestamp = new Date(1000L * (0xFFFFFFFFL & buffer.getInt()));
      this.dataSize = 0xFFFF & buffer.getShort();
      this.roundtripDelay = buffer.getInt();
    } catch (Exception e) {
      e.printStackTrace();
      throw new IllegalArgumentException();
    } 
  }
  
  protected byte[] getMessageDataBytes() throws MessageFormatException {
    return this.data;
  }
  
  public Date getPingTimestamp() {
    return this.pingTimestamp;
  }
  
  public int getDataSize() {
    return this.dataSize;
  }
  
  public int getRoundtripDelay() {
    return this.roundtripDelay;
  }
  
  public byte[] getUueid() {
    return this.uueid;
  }
  
  public int getPingId() {
    return this.pingId;
  }
  
  public synchronized PSCommandPingEndDevice getPing() {
    return this.ping;
  }
  
  public synchronized void setPing(PSCommandPingEndDevice ping) {
    this.ping = ping;
  }
}
