package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.StandardCharsets;

public class PSCommandATCommand extends PSCommand {
  public static final int BYTES_FOR_AT_COMMAND_LENGTH = 2;
  
  private String atCommand;
  
  public PSCommandATCommand(String atCommand, PSCommandDelegate delegate) {
    super((byte[])null, (short)17, delegate);
    this.atCommand = atCommand.trim().replaceAll("(\\r|\\n)", "");
    setUpData(this.atCommand);
    CharsetEncoder asciiEncoder = StandardCharsets.US_ASCII.newEncoder();
    boolean canEncode = asciiEncoder.canEncode(atCommand);
    if (!canEncode)
      throw new IllegalArgumentException("Only US-ASCII chars allowed"); 
  }
  
  private void setUpData(String atCmd) {
    byte[] atCmdBytes = this.atCommand.getBytes(StandardCharsets.US_ASCII);
    ByteBuffer buf = ByteBuffer.allocate(2 + atCmdBytes.length).order(ByteOrder.LITTLE_ENDIAN);
    buf.putShort((short)atCmdBytes.length);
    buf.put(atCmdBytes);
    this.data = buf.array();
  }
  
  public void setAtCommand(String atCmd) {
    this.atCommand = atCmd;
    setUpData(this.atCommand);
  }
  
  public String getAtCommand() {
    return this.atCommand;
  }
  
  public String getTypeString() {
    return "AT Command";
  }
  
  public String getContentString() {
    return this.atCommand;
  }
}
