package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.wp.model.bsparameters.BSParameter;

public class PSCommandParameter extends PSCommandWithOperations {
  public static final int BYTES_FOR_VALUE = 4;
  
  public static final int BYTES_FOR_INDEX = 2;
  
  protected byte[] value;
  
  protected String name;
  
  protected String uiName;
  
  protected int index;
  
  public PSCommandParameter(PSCommandDelegate delegate, String name, String uiName, byte operation, byte[] value, int index) {
    super((byte[])null, (short)16, delegate, operation);
    this.name = name;
    this.uiName = uiName;
    this.value = value;
    this.index = index;
    if (this.value == null) {
      this.value = new byte[0];
    } else if (value != null && value.length != 4) {
      throw new IllegalArgumentException();
    } 
    setUpData();
  }
  
  public PSCommandParameter(PSCommandDelegate delegate, BSParameter parameter, byte operation) {
    this(delegate, parameter.getName(), parameter.getUiName(), operation, parameter.getValueInBytes(), parameter.getIndex());
  }
  
  private void setUpData() {
    ByteBuffer buffer = ByteBuffer.allocate(35 + this.value.length).order(ByteOrder.LITTLE_ENDIAN);
    buffer.put(getStringBytesWithPadding(this.name))
      .putShort((short)this.index)
      .put(this.operation)
      .put(this.value);
    this.data = buffer.array();
  }
  
  public String getTypeString() {
    String str = super.getTypeString();
    str = str.concat(" ").concat(this.uiName);
    return str;
  }
  
  public String getContentString() {
    String str = "";
    if (this.value != null)
      try {
        int intVal = ByteBuffer.wrap(this.value).order(ByteOrder.LITTLE_ENDIAN).getInt();
        str = Long.toString(0xFFFFFFFFL & intVal);
      } catch (Exception exception) {} 
    return str;
  }
  
  public byte[] getValue() {
    return this.value;
  }
  
  public Long getLongValue() {
    if (this.value != null) {
      int intVal = ByteBuffer.wrap(this.value).order(ByteOrder.LITTLE_ENDIAN).getInt();
      return Long.valueOf(intVal);
    } 
    return null;
  }
}
