package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import ubiik.utils.DatatypeConverter;
import ubiik.wp.model.messages.PSPong;

public class PSCommandPingEndDevice extends PSCommand {
  public static final int BYTES_FOR_TARGET = 16;
  
  public static final int BYTES_FOR_PAYLOAD_SIZE = 2;
  
  public static final int BYTES_FOR_PING_ID = 4;
  
  public static final int MAX_PAYLOAD_SIZE = 65535;
  
  public static final int MIN_PAYLOAD_SIZE = 0;
  
  private byte[] target;
  
  private int payloadBytesSize;
  
  private int pingId;
  
  private PSPong pong;
  
  public PSCommandPingEndDevice(PSCommandDelegate delegate, byte[] target, int payloadSize, int pingId) {
    super((byte[])null, (short)32, delegate);
    if (target.length != 16)
      throw new IllegalArgumentException(); 
    if (payloadSize > 65535 || payloadSize < 0)
      throw new IllegalArgumentException("Invalid payload Size"); 
    this.payloadBytesSize = payloadSize;
    this.target = target;
    this.pingId = pingId;
    setUpData();
    this.pong = null;
  }
  
  public void setUpData() {
    this
      
      .data = ByteBuffer.allocate(22).order(ByteOrder.LITTLE_ENDIAN).put(this.target).putShort((short)this.payloadBytesSize).putInt(this.pingId).array();
  }
  
  public String getTypeString() {
    return "Ping";
  }
  
  public String getContentString() {
    return DatatypeConverter.printHexBinary(this.target);
  }
  
  public int getPingId() {
    return this.pingId;
  }
  
  public synchronized PSPong getPong() {
    return this.pong;
  }
  
  public synchronized void setPong(PSPong pong) {
    this.pong = pong;
  }
  
  public byte[] getTarget() {
    return this.target;
  }
}
