package ubiik.wp.model.messages.pscommands;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class PSCommandPutFileEx extends PSCommand {
  private static final int FILE_NAME_SIZE = 256;
  
  private static final int FILE_USERNAME_SIZE = 32;
  
  private static final int FILE_GROUPNAME_SIZE = 32;
  
  private static final int PUT_FILE_HEADER_SIZE = 332;
  
  private byte[] filename;
  
  private byte[] userName;
  
  private byte[] groupName;
  
  private int permissions;
  
  private long fragOffset;
  
  private byte[] fragData;
  
  public PSCommandPutFileEx(String filename, String userName, String groupName, int permissions, long fragOffset, byte[] fragData, PSCommandDelegate delegate) {
    super((byte[])null, (short)47, delegate);
    this.filename = filename.getBytes();
    if (this.filename.length > 256)
      throw new IllegalArgumentException("Filename can't be longer than 256"); 
    this.filename = Arrays.copyOf(this.filename, 256);
    this.userName = userName.getBytes();
    if (this.userName.length > 32)
      throw new IllegalArgumentException("Username can't be longer than 32"); 
    this.userName = Arrays.copyOf(this.userName, 32);
    this.groupName = groupName.getBytes();
    if (this.groupName.length > 32)
      throw new IllegalArgumentException("Group name can't be longer than 32"); 
    this.groupName = Arrays.copyOf(this.groupName, 32);
    this.permissions = permissions;
    this.fragOffset = fragOffset;
    this.fragData = fragData;
    setUpData();
  }
  
  private void setUpData() {
    ByteBuffer buf = ByteBuffer.allocate(332 + this.fragData.length).order(ByteOrder.LITTLE_ENDIAN);
    buf.put(this.filename);
    buf.put(this.userName);
    buf.put(this.groupName);
    buf.putInt(this.permissions);
    buf.putInt((int)this.fragOffset);
    buf.putInt(this.fragData.length);
    buf.put(this.fragData);
    this.data = buf.array();
  }
  
  public String getTypeString() {
    return "Put File Command";
  }
  
  public String getContentString() {
    return new String(this.filename);
  }
}
