package ubiik.wp.model.messages.pscommands;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;

public abstract class PSCommandWithOperations extends PSCommand {
  public static final int CHARS_FOR_NAME = 32;
  
  public static final int PARAMETER_GET = 0;
  
  public static final int PARAMETER_SET = 1;
  
  public static final String[] PARAMETER_NAMES = new String[] { "GET", "SET" };
  
  public static final int BYTES_FOR_OPERATION = 1;
  
  protected byte operation;
  
  public PSCommandWithOperations(byte[] messageData, short commandType, PSCommandDelegate delegate, byte operation) {
    super(messageData, commandType, delegate);
    this.operation = operation;
  }
  
  public synchronized byte getOperation() {
    return this.operation;
  }
  
  public boolean isSet() {
    return (this.operation == 1);
  }
  
  public boolean isGet() {
    return (this.operation == 0);
  }
  
  public String getTypeString() {
    String str = "";
    int idx = 0xFF & this.operation;
    if (idx < PARAMETER_NAMES.length)
      str = str.concat(PARAMETER_NAMES[idx]); 
    return str;
  }
  
  protected static byte[] getStringBytesWithPadding(String str) {
    byte[] strBytes = str.getBytes(StandardCharsets.US_ASCII);
    return Arrays.copyOf(strBytes, 32);
  }
}
