package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.StandardCharsets;
import ubiik.wp.util.ApplicationLogger;

public class PSCommandResponseATCommand extends PSCommandResponse {
  private String atCommandResponse;
  
  public PSCommandResponseATCommand(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)17);
    byte[] respData = null;
    if (getErrorCode() == 0) {
      try {
        respData = getCommandResponseData();
        if (respData.length < 2)
          throw new IllegalArgumentException("Command Response Data length is: " + respData.length); 
        ByteBuffer buffer = ByteBuffer.wrap(respData).order(ByteOrder.LITTLE_ENDIAN);
        int respLength = 0xFFFF & buffer.getShort();
        CharsetDecoder decoder = StandardCharsets.ISO_8859_1.newDecoder();
        this.atCommandResponse = decoder.decode(buffer).toString();
        if (respLength != this.atCommandResponse.length())
          ApplicationLogger.debug("AT command response reported length and actual length do not match"); 
      } catch (Exception e) {
        throw new IllegalArgumentException(e.getMessage());
      } 
    } else {
      this.atCommandResponse = null;
    } 
  }
  
  public String getAtCommandResponse() {
    return this.atCommandResponse;
  }
  
  public String getTypeString() {
    return "Response to AT Command";
  }
  
  public String getContentString() {
    return getAtCommandResponse();
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseATCommand)
      return this.atCommandResponse.equalsIgnoreCase(((PSCommandResponseATCommand)other).getAtCommandResponse()); 
    return false;
  }
}
