package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class PSCommandResponseEDRASlots extends PSCommandResponse {
  protected short numRaSlots;
  
  protected short numRaSlotsSlowMcs;
  
  public PSCommandResponseEDRASlots(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)36);
    this.numRaSlots = 0;
    this.numRaSlotsSlowMcs = 0;
    ByteBuffer buffer = ByteBuffer.wrap(commandData, 1, commandData.length - 1).order(ByteOrder.LITTLE_ENDIAN);
    if (getErrorCode() == 0 && 
      buffer.remaining() == 4) {
      this.numRaSlots = buffer.getShort();
      this.numRaSlotsSlowMcs = buffer.getShort();
    } 
  }
  
  public String getTypeString() {
    return "Response to SET/GET RA Setting";
  }
  
  public String getContentString() {
    StringBuilder sb = new StringBuilder();
    if (this.numRaSlots != -1)
      sb.append("RA Slots " + this.numRaSlots); 
    if (this.numRaSlotsSlowMcs != -1)
      sb.append(" RA Slots (slow MCS) " + this.numRaSlotsSlowMcs); 
    return sb.toString();
  }
  
  public short getRASlots() {
    return this.numRaSlots;
  }
  
  public short getRASlotsSlowMcs() {
    return this.numRaSlotsSlowMcs;
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseEDRASlots) {
      PSCommandResponseEDRASlots otherSlots = (PSCommandResponseEDRASlots)other;
      return (this.numRaSlots == otherSlots.getRASlots() && this.numRaSlotsSlowMcs == otherSlots
        .getRASlotsSlowMcs());
    } 
    return false;
  }
}
