package ubiik.wp.model.messages.pscommands.responses;

import java.util.Arrays;

public class PSCommandResponseFPGAReadRegister extends PSCommandResponse {
  protected byte registerIdx;
  
  protected byte[] registerContent;
  
  public PSCommandResponseFPGAReadRegister(int messageId, int commandId, byte[] commandData, short commandType) {
    super(messageId, commandId, commandData, commandType);
    if (getErrorCode() == 0) {
      try {
        this.registerIdx = commandData[1];
        this.registerContent = Arrays.copyOfRange(commandData, 2, commandData.length);
      } catch (Exception e) {
        throw new IllegalArgumentException();
      } 
      if (this.registerContent.length == 0 || this.registerContent.length % 4 != 0)
        throw new IllegalArgumentException(); 
    } else {
      this.registerIdx = -1;
      this.registerContent = null;
    } 
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseFPGAReadRegister) {
      PSCommandResponseFPGAReadRegister otherFpga = (PSCommandResponseFPGAReadRegister)other;
      return (this.registerIdx == otherFpga.getRegisterIdx() && 
        Arrays.equals(this.registerContent, ((PSCommandResponseFPGAReadRegister)other).getRegisterContent()));
    } 
    return false;
  }
  
  public byte getRegisterIdx() {
    return this.registerIdx;
  }
  
  public byte[] getRegisterContent() {
    return this.registerContent;
  }
  
  public String getTypeString() {
    return "Resp to FPGA Read Register";
  }
  
  public String getContentString() {
    String content;
    if (getErrorCode() == 0) {
      content = "" + this.registerIdx + " " + this.registerIdx;
    } else {
      content = "Error";
    } 
    return content;
  }
}
