package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class PSCommandResponseGetScheduleTransfer extends PSCommandResponse {
  private byte[] deviceUuid;
  
  private byte flag;
  
  private int interval;
  
  private int window;
  
  public PSCommandResponseGetScheduleTransfer(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)19);
    this.deviceUuid = new byte[16];
    this.flag = -1;
    this.interval = -1;
    this.window = -1;
    ByteBuffer buffer = ByteBuffer.wrap(commandData, 1, commandData.length - 1).order(ByteOrder.LITTLE_ENDIAN);
    if (getErrorCode() == 0) {
      try {
        buffer.get(this.deviceUuid);
        this.flag = buffer.get();
        if (this.flag == 1) {
          this.interval = 0xFFFF & buffer.getShort();
          this.window = 0xFFFF & buffer.getShort();
        } 
      } catch (Exception e) {
        throw new IllegalArgumentException();
      } 
    } else {
      this.deviceUuid = null;
    } 
  }
  
  public String getTypeString() {
    return "Response to Get Schedule Transfer";
  }
  
  public String getContentString() {
    if (this.flag == 1) {
      StringBuilder str = new StringBuilder("Set interval = ");
      str.append(this.interval).append(", window = ").append(this.window);
      return str.toString();
    } 
    return "release";
  }
  
  public byte[] getDeviceUuid() {
    return this.deviceUuid;
  }
  
  public byte getFlag() {
    return this.flag;
  }
  
  public int getInterval() {
    return this.interval;
  }
  
  public int getWindow() {
    return this.window;
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseGetScheduleTransfer)
      return (Arrays.equals(this.deviceUuid, ((PSCommandResponseGetScheduleTransfer)other).getDeviceUuid()) && this.flag == ((PSCommandResponseGetScheduleTransfer)other)
        .getFlag() && this.interval == ((PSCommandResponseGetScheduleTransfer)other)
        .getInterval() && this.window == ((PSCommandResponseGetScheduleTransfer)other)
        .getWindow()); 
    return false;
  }
}
