package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class PSCommandResponseMCSSetting extends PSCommandResponse {
  protected short uplinkVal;
  
  protected short downlinkVal;
  
  public PSCommandResponseMCSSetting(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)34);
    this.uplinkVal = -1;
    this.downlinkVal = -1;
    ByteBuffer buffer = ByteBuffer.wrap(commandData, 1, commandData.length - 1).order(ByteOrder.LITTLE_ENDIAN);
    if (getErrorCode() == 0 && 
      buffer.remaining() == 4) {
      this.downlinkVal = buffer.getShort();
      this.uplinkVal = buffer.getShort();
    } 
  }
  
  public String getTypeString() {
    return "Response to SET/GET MCS Setting";
  }
  
  public String getContentString() {
    StringBuilder sb = new StringBuilder();
    if (this.downlinkVal != -1)
      sb.append("Downlink MCS " + this.downlinkVal); 
    if (this.uplinkVal != -1)
      sb.append(" Uplink MCS " + this.uplinkVal); 
    return sb.toString();
  }
  
  public short getUplinkVal() {
    return this.uplinkVal;
  }
  
  public short getDownlinkVal() {
    return this.downlinkVal;
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseMCSSetting)
      return (this.downlinkVal == ((PSCommandResponseMCSSetting)other).getDownlinkVal() && this.uplinkVal == ((PSCommandResponseMCSSetting)other)
        .getUplinkVal()); 
    return false;
  }
}
