package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class PSCommandResponseMCSValue extends PSCommandResponse {
  private byte downlinkMCSValue;
  
  private byte uplinkMCSValue;
  
  private byte uplinkMCSValueNB;
  
  public PSCommandResponseMCSValue(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)35);
    this.downlinkMCSValue = -1;
    this.uplinkMCSValue = -1;
    this.uplinkMCSValueNB = -1;
    ByteBuffer buffer = ByteBuffer.wrap(commandData, 1, commandData.length - 1).order(ByteOrder.LITTLE_ENDIAN);
    if (getErrorCode() == 0 && 
      buffer.remaining() == 3) {
      this.downlinkMCSValue = buffer.get();
      this.uplinkMCSValue = buffer.get();
      this.uplinkMCSValueNB = buffer.get();
    } 
  }
  
  public String getTypeString() {
    return "Response to SET/GET MCS Setting";
  }
  
  public String getContentString() {
    StringBuilder sb = new StringBuilder();
    if (this.downlinkMCSValue != -1)
      sb.append("Downlink MCS " + this.downlinkMCSValue); 
    if (this.uplinkMCSValue != -1)
      sb.append(" Uplink MCS " + this.uplinkMCSValue); 
    if (this.uplinkMCSValueNB != -1)
      sb.append(" Uplink MCS NB " + this.uplinkMCSValueNB); 
    return sb.toString();
  }
  
  public synchronized byte getDownlinkMCSValue() {
    return this.downlinkMCSValue;
  }
  
  public synchronized byte getUplinkMCSValue() {
    return this.uplinkMCSValue;
  }
  
  public synchronized byte getUplinkMCSValueNB() {
    return this.uplinkMCSValueNB;
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseMCSValue)
      return (this.downlinkMCSValue == ((PSCommandResponseMCSValue)other).getDownlinkMCSValue() && this.uplinkMCSValue == ((PSCommandResponseMCSValue)other)
        .getUplinkMCSValue() && this.uplinkMCSValueNB == ((PSCommandResponseMCSValue)other)
        .getUplinkMCSValueNB()); 
    return false;
  }
}
