package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class PSCommandResponseReadBlacklistedChannels extends PSCommandResponse {
  protected int[] channels;
  
  public PSCommandResponseReadBlacklistedChannels(int messageId, int commandId, byte[] commandData, short commandType) {
    super(messageId, commandId, commandData, commandType);
    byte[] channelsData = null;
    if (getErrorCode() == 0) {
      try {
        channelsData = getCommandResponseData();
        if (channelsData.length == 0 || channelsData.length % 2 != 0)
          throw new IllegalArgumentException(); 
        ByteBuffer buffer = ByteBuffer.wrap(channelsData).order(ByteOrder.LITTLE_ENDIAN);
        int totalChannels = buffer.getShort();
        this.channels = new int[totalChannels];
        for (int i = 0; i < totalChannels; i++)
          this.channels[i] = buffer.getShort(); 
      } catch (Exception e) {
        throw new IllegalArgumentException();
      } 
    } else {
      this.channels = null;
    } 
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseReadBlacklistedChannels) {
      int[] sortedChannels = Arrays.copyOf(this.channels, this.channels.length);
      Arrays.sort(sortedChannels);
      int[] sortedOthers = Arrays.copyOf(((PSCommandResponseReadBlacklistedChannels)other).getChannels(), (((PSCommandResponseReadBlacklistedChannels)other)
          .getChannels()).length);
      Arrays.sort(sortedOthers);
      return Arrays.equals(sortedChannels, sortedOthers);
    } 
    return false;
  }
  
  public int[] getChannels() {
    return this.channels;
  }
  
  public String getTypeString() {
    return "Response to Read Blacklisted Channel";
  }
  
  public String getContentString() {
    String resp = "";
    for (int channel : this.channels)
      resp = resp + resp + " "; 
    return resp;
  }
}
