package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class PSCommandResponseReadMulticastGroups extends PSCommandResponse {
  private HashMap<Integer, List<byte[]>> multicastGroups;
  
  private HashMap<Integer, List<Byte>> devicesStatus;
  
  public PSCommandResponseReadMulticastGroups(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)3);
    this.devicesStatus = new HashMap<>();
    ByteBuffer buffer = ByteBuffer.wrap(commandData, 1, commandData.length - 1).order(ByteOrder.LITTLE_ENDIAN);
    if (getErrorCode() == 0) {
      this.multicastGroups = new HashMap<>();
      try {
        int totalGroups = 0xFFFF & buffer.getShort();
        loadMulticastGroupsFromData(buffer, totalGroups, this.multicastGroups, this.devicesStatus);
      } catch (Exception e) {
        this.multicastGroups = null;
        throw new IllegalArgumentException();
      } 
    } else {
      this.multicastGroups = null;
    } 
  }
  
  public static final void loadMulticastGroupsFromData(ByteBuffer buffer, int totalGroups, HashMap<Integer, List<byte[]>> multicastGroups, HashMap<Integer, List<Byte>> devicesStatus) {
    try {
      for (int i = 0; i < totalGroups; i++) {
        int groupId = 0xFFFF & buffer.getShort();
        int totalDevices = 0xFFFF & buffer.getShort();
        ArrayList<byte[]> devList = (ArrayList)new ArrayList<>();
        ArrayList<Byte> devStatusList = new ArrayList<>();
        for (int j = 0; j < totalDevices; j++) {
          byte[] uuid = new byte[16];
          buffer.get(uuid);
          devList.add(uuid);
          devStatusList.add(Byte.valueOf(buffer.get()));
        } 
        multicastGroups.put(Integer.valueOf(groupId), devList);
        devicesStatus.put(Integer.valueOf(groupId), devStatusList);
      } 
    } catch (Exception e) {
      multicastGroups = null;
      devicesStatus = null;
      e.printStackTrace();
      throw new IllegalArgumentException();
    } 
  }
  
  public String getTypeString() {
    return "Response to read Multicast Groups";
  }
  
  public String getContentString() {
    String resp = "";
    for (Integer i : this.multicastGroups.keySet())
      resp = resp + resp + " "; 
    return resp;
  }
  
  public HashMap<Integer, List<byte[]>> getMulticastGroups() {
    return this.multicastGroups;
  }
  
  public HashMap<Integer, List<Byte>> getDevicesStatusInMulticastGroups() {
    return this.devicesStatus;
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseReadMulticastGroups) {
      boolean responseEqual = false;
      HashMap<Integer, List<byte[]>> otherGroups = ((PSCommandResponseReadMulticastGroups)other).getMulticastGroups();
      HashMap<Integer, List<Byte>> otherStatus = ((PSCommandResponseReadMulticastGroups)other).getDevicesStatusInMulticastGroups();
      if (this.multicastGroups.size() == otherGroups.size())
        for (Map.Entry<Integer, List<byte[]>> entry : this.multicastGroups.entrySet()) {
          boolean listEqual = false;
          boolean statusEqual = false;
          List<byte[]> otherList = otherGroups.get(entry.getKey());
          if (otherList.size() == ((List)entry.getValue()).size()) {
            boolean found = false;
            for (int i = 0; i < ((List)entry.getValue()).size(); i++) {
              for (int j = 0; j < otherList.size() && !found; j++) {
                if (Arrays.equals(((List<byte[]>)entry.getValue()).get(i), otherList.get(j))) {
                  found = true;
                  if ((this.devicesStatus.get(entry.getKey())).get(i) == (otherStatus
                    .get(entry.getKey())).get(j))
                    statusEqual = true; 
                  break;
                } 
              } 
              listEqual = (found && statusEqual);
              if (!listEqual)
                break; 
            } 
          } 
          responseEqual = listEqual & statusEqual;
          if (!responseEqual)
            break; 
        }  
      return responseEqual;
    } 
    return false;
  }
}
