package ubiik.wp.model.messages.pscommands.responses;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class PSCommandResponseRunMode extends PSCommandResponse {
  private Integer mode;
  
  private Integer subMode;
  
  public PSCommandResponseRunMode(int messageId, int commandId, byte[] commandData) {
    super(messageId, commandId, commandData, (short)20);
    this.mode = null;
    ByteBuffer buffer = ByteBuffer.wrap(commandData, 1, commandData.length - 1).order(ByteOrder.LITTLE_ENDIAN);
    if (getErrorCode() == 0 && (
      commandData.length - 1 == 4 || commandData.length - 1 == 2)) {
      try {
        this.mode = Integer.valueOf(0xFFFF & buffer.getShort());
      } catch (Exception e) {
        throw new IllegalArgumentException();
      } 
      try {
        this.subMode = Integer.valueOf(0xFFFF & buffer.getShort());
      } catch (Exception e) {
        this.subMode = Integer.valueOf(0);
      } 
    } 
  }
  
  public Integer getMode() {
    return this.mode;
  }
  
  public Integer getSubMode() {
    return this.subMode;
  }
  
  public String getTypeString() {
    return "Response to Parameter Command";
  }
  
  public String getContentString() {
    if (this.mode != null)
      return this.mode.toString(); 
    return "";
  }
  
  public boolean doCompareResponseData(PSCommandResponse other) {
    if (other instanceof PSCommandResponseRunMode)
      return (this.mode == ((PSCommandResponseRunMode)other).getMode() && this.subMode == ((PSCommandResponseRunMode)other)
        .getSubMode()); 
    return false;
  }
}
