package ubiik.wp.model.sdr;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class SDRParameterUInt32 extends SDRParameter {
  public static final String SDR_PARAM_VALUE_TYPE = "uint_32";
  
  public static final long UINT32_MAX_VAL = 4294967295L;
  
  public static final long UINT32_MIN_VAL = 0L;
  
  protected long value;
  
  private long defaultValue;
  
  private long max;
  
  private long min;
  
  private long step;
  
  public SDRParameterUInt32(int id, String name, String description, String category, long value, long defaultValue, long max, long min, long step) {
    super(id, name, description, category);
    this.value = value;
    this.defaultValue = defaultValue;
    this.max = max;
    this.min = min;
    this.step = step;
  }
  
  public SDRParameterUInt32(int id, String name, String description, String category, long defaultValue) {
    this(id, name, description, category, defaultValue, defaultValue, 4294967295L, 0L, 0L);
  }
  
  public synchronized boolean validateValue(String val) {
    boolean valid = true;
    try {
      long longVal = Long.parseLong(val);
      if (longVal > this.max || longVal < this.min)
        valid = false; 
      if (this.step > 0L && 
        longVal % this.step != 0L)
        valid = false; 
    } catch (Exception e) {
      valid = false;
    } 
    return valid;
  }
  
  public synchronized boolean setValuFromString(String str) {
    boolean valid = validateValue(str);
    if (valid)
      this.value = Long.parseLong(str); 
    return valid;
  }
  
  public synchronized byte[] getValueInBytes() {
    return ByteBuffer.allocate(4)
      .order(ByteOrder.LITTLE_ENDIAN)
      .putInt((int)this.value)
      .array();
  }
  
  public byte getDataTypeCode() {
    return 1;
  }
  
  public String getMinStr() {
    return Long.toString(this.min);
  }
  
  public String getMaxStr() {
    return Long.toString(this.max);
  }
  
  public String getStepStr() {
    return Long.toString(this.step);
  }
  
  public String getValueStr() {
    return Long.toString(this.value);
  }
  
  public String getDefaultStr() {
    return Long.toString(this.defaultValue);
  }
}
