package ubiik.wp.model.sdr;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class SDRParameterUint8 extends SDRParameterInt {
  public static final String SDR_PARAM_VALUE_TYPE = "uint8";
  
  public static final int UINT8_MAX_VAL = 255;
  
  public static final int UINT8_MIN_VAL = 0;
  
  public SDRParameterUint8(int id, String name, String description, String category, int value, int defaultValue, int max, int min, int step) {
    super(id, name, description, category, value, defaultValue, max, min, step);
    if (value > 255 || defaultValue > 255 || max > 255 || min > 255 || value < 0 || defaultValue < 0 || max < 0 || min < 0)
      throw new IllegalArgumentException(); 
  }
  
  public SDRParameterUint8(int id, String name, String description, String category, int defaultValue) {
    this(id, name, description, category, defaultValue, defaultValue, 2147483647, -2147483648, 0);
  }
  
  public synchronized byte[] getValueInBytes() {
    return ByteBuffer.allocate(1).order(ByteOrder.LITTLE_ENDIAN).put((byte)(0xFF & this.value)).array();
  }
  
  public byte getDataTypeCode() {
    return 0;
  }
}
