package ubiik.wp.model.sdr;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class SDRparameterDouble extends SDRParameter {
  public static final String SDR_PARAM_VALUE_TYPE = "double";
  
  private double value;
  
  private double defaultValue;
  
  private double max;
  
  private double min;
  
  private double step;
  
  public SDRparameterDouble(int id, String name, String description, String category, double value, double defaultValue, double max, double min, double step) {
    super(id, name, description, category);
    this.value = value;
    this.defaultValue = defaultValue;
    this.max = max;
    this.min = min;
    this.step = step;
  }
  
  public SDRparameterDouble(int id, String name, String description, String category, double defaultValue) {
    this(id, name, description, category, defaultValue, defaultValue, Double.MAX_VALUE, Double.MIN_VALUE, 0.0D);
  }
  
  public boolean validateValue(String val) {
    boolean valid = true;
    try {
      double dVal = Double.parseDouble(val);
      if (dVal > this.max || dVal < this.min)
        valid = false; 
      if (this.step > 0.0D) {
        int div = (int)(dVal / this.step);
        if (div * this.step < dVal)
          valid = false; 
      } 
    } catch (Exception e) {
      valid = false;
    } 
    return valid;
  }
  
  public boolean setValuFromString(String str) {
    boolean valid = validateValue(str);
    if (valid)
      this.value = Double.parseDouble(str); 
    return valid;
  }
  
  public byte[] getValueInBytes() {
    return ByteBuffer.allocate(8).order(ByteOrder.LITTLE_ENDIAN).putDouble(this.value).array();
  }
  
  public byte getDataTypeCode() {
    return 1;
  }
  
  public String getMinStr() {
    return Double.toString(this.min);
  }
  
  public String getMaxStr() {
    return Double.toString(this.max);
  }
  
  public String getStepStr() {
    return Double.toString(this.step);
  }
  
  public String getValueStr() {
    return Double.toString(this.value);
  }
  
  public String getDefaultStr() {
    return Double.toString(this.defaultValue);
  }
}
