package ubiik.wp.persistence;

import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import ubiik.utils.DatatypeConverter;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.capacity.MessageCapacityTest;
import ubiik.wp.model.capacity.MessageCapacityTestFactory;
import ubiik.wp.model.devices.UUEndDevice;

public class MessageCapacityTestJSONParser {
  private static final String KEY_TESTS = "capacity_tests";
  
  private static final String KEY_CHANNEL = "channel";
  
  private static final String KEY_END_DEVICE = "device";
  
  private static final String KEY_INTERVAL = "interval";
  
  private static final String KEY_DATA_SIZE = "data_size";
  
  private static final String KEY_TOTAL_MSG = "total_msg";
  
  private static final String KEY_CHANNEL_DATA_ACK = "data_ack";
  
  private static final String KEY_CHANNEL_DATA_UNACK = "data_unack";
  
  private static final String KEY_CHANNEL_CONTROL = "control";
  
  private static final Map<String, Integer> CHANNEL_VALUES = new HashMap<>();
  
  static {
    CHANNEL_VALUES.put("data_ack", Integer.valueOf(0));
    CHANNEL_VALUES.put("data_unack", Integer.valueOf(1));
    CHANNEL_VALUES.put("control", Integer.valueOf(2));
  }
  
  public static final List<MessageCapacityTest> parseJSON(InputStreamReader in, MessageController messageController, DevicesController devicesController) throws IllegalArgumentException {
    List<MessageCapacityTest> tests = new LinkedList<>();
    JSONParser jsonParser = new JSONParser();
    try {
      JSONObject root = (JSONObject)jsonParser.parse(in);
      JSONArray groupsArray = (JSONArray)root.get("capacity_tests");
      for (Object object : groupsArray) {
        JSONObject jObject = (JSONObject)object;
        Integer dataSize = Integer.valueOf(((Long)jObject.get("data_size")).intValue());
        Integer interval = Integer.valueOf(((Long)jObject.get("interval")).intValue());
        String device = (String)jObject.get("device");
        String channelStr = (String)jObject.get("channel");
        Integer channel = CHANNEL_VALUES.get(channelStr);
        Integer totalMsg = Integer.valueOf(((Long)jObject.get("total_msg")).intValue());
        byte[] uueid = DatatypeConverter.parseHexBinary(device);
        UUEndDevice ed = devicesController.getDeviceForId(uueid);
        if (ed == null)
          ed = devicesController.addDevice(uueid); 
        if (ed != null && channel != null) {
          MessageCapacityTest mct = MessageCapacityTestFactory.createTest(interval
              .intValue(), dataSize.intValue(), ed, channel.intValue(), messageController, devicesController, totalMsg.intValue());
          if (mct != null)
            tests.add(mct); 
        } 
      } 
    } catch (Exception e) {
      throw new IllegalArgumentException("Failed to parse Capacity Test");
    } 
    return tests;
  }
  
  public static final JSONObject capacityTestsToJson(Collection<MessageCapacityTest> capacityTests) {
    JSONObject root = new JSONObject();
    JSONArray groupsArray = new JSONArray();
    for (MessageCapacityTest capacityTest : capacityTests)
      groupsArray.add(capacityTestToJson(capacityTest)); 
    root.put("capacity_tests", groupsArray);
    return root;
  }
  
  public static final JSONObject capacityTestToJson(MessageCapacityTest capacityTest) {
    JSONObject groupObj = new JSONObject();
    Integer channelCode = Integer.valueOf(capacityTest.getChannelCode());
    String channelKey = null;
    for (Map.Entry<String, Integer> entry : CHANNEL_VALUES.entrySet()) {
      if (entry.getValue() == channelCode)
        channelKey = entry.getKey(); 
    } 
    if (channelKey != null) {
      groupObj.put("channel", channelKey);
      groupObj.put("device", capacityTest.getEndDevice().getUUIDString());
      groupObj.put("interval", Integer.valueOf(capacityTest.getInterval()));
      groupObj.put("data_size", Integer.valueOf(capacityTest.getDataSize()));
      groupObj.put("total_msg", Long.valueOf(capacityTest.getMessagesToSend()));
    } else {
      throw new IllegalArgumentException("Invalid Channel Key");
    } 
    return groupObj;
  }
  
  public static final void capacityTestsToFile(Collection<MessageCapacityTest> capacityTests, String path) throws IOException {
    JSONObject jsonCapacityTest = capacityTestsToJson(capacityTests);
    FileWriter file = new FileWriter(path);
    Throwable throwable = null;
    try {
      file.write(jsonCapacityTest.toJSONString());
    } catch (Throwable throwable1) {
      throwable = throwable1 = null;
      throw new AssertionError(throwable1);
    } finally {
      if (throwable != null) {
        try {
          file.close();
        } catch (Throwable throwable1) {
          throwable.addSuppressed(throwable1);
        } 
      } else {
        file.close();
      } 
    } 
  }
}
