package ubiik.wp.psconnection;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public abstract class PSConnection {
  private static final int BYTES_PKT_LEN = 4;
  
  private static final int BYTES_CRC = 2;
  
  protected short crc16(byte[] data) {
    int length = data.length;
    int i = 0;
    int crc = 65535;
    while (length-- > 0) {
      int x = crc >>> 8 ^ 0xFF & data[i++];
      x ^= x >>> 4;
      crc = 0xFFFF & (crc << 8 ^ x << 12 ^ x << 5 ^ x);
    } 
    return (short)(0xFFFF & crc);
  }
  
  protected byte[] buildPkt(byte[] data) {
    ByteBuffer buffer = ByteBuffer.allocate(data.length + 4 + 2);
    buffer.order(ByteOrder.LITTLE_ENDIAN);
    short crc = crc16(data);
    int len = data.length;
    buffer.putInt(len);
    buffer.put(data);
    buffer.putShort(crc);
    return buffer.array();
  }
}
