package ubiik.wp.psconnection;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.atomic.AtomicBoolean;
import ubiik.wp.util.ApplicationLogger;

public abstract class PSConnectionManager implements PSConnectionDelegate {
  public static final int PSCONNECT_TIMEOUT = 5000;
  
  public static final int PS_AUTHENTICATE_TIMEOUT = 5000;
  
  protected String host;
  
  protected int port;
  
  protected PSConnectionManagerDelegate delegate;
  
  protected AtomicBoolean authenticated;
  
  protected AtomicBoolean receivedBaseStationId;
  
  protected BlockingQueue<byte[]> inMessages;
  
  public PSConnectionManager(String host, int port, PSConnectionManagerDelegate delegate, BlockingQueue<byte[]> queue) {
    this.delegate = delegate;
    this.host = host;
    this.port = port;
    this.authenticated = new AtomicBoolean(false);
    this.receivedBaseStationId = new AtomicBoolean(false);
    this.inMessages = queue;
  }
  
  public abstract void connect() throws PSConnectionException;
  
  public abstract void disconnect();
  
  public abstract void sendBytes(byte[] paramArrayOfbyte);
  
  public AtomicBoolean isAuthenticated() {
    return this.authenticated;
  }
  
  public boolean getReceivedBaseStationId() {
    return this.receivedBaseStationId.get();
  }
  
  public abstract void receivedBytes(byte[] paramArrayOfbyte);
  
  public void connectionFailed(PSConnectionErrorCode errorCode) {
    if (this.delegate != null)
      this.delegate.connectionFailed(errorCode); 
    ApplicationLogger.debug("connection failed ERR: " + errorCode);
  }
  
  public String getHost() {
    return this.host;
  }
  
  public BlockingQueue<byte[]> getInMessagesQueue() {
    return this.inMessages;
  }
}
