package ubiik.wp.psconnection.auth;

import java.security.GeneralSecurityException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import ubiik.wp.model.messages.PSLogMessage;
import ubiik.wp.model.messages.pscommands.PSCommandParameter;
import ubiik.wp.model.messages.pscommands.PSCommandReadBlacklistedChannels;
import ubiik.wp.model.messages.pscommands.PSCommandReadMulticastGroups;
import ubiik.wp.model.messages.pscommands.PSCommandSetBaseChannel;
import ubiik.wp.model.messages.pscommands.PSCommandSetSIB0and1;

public abstract class PSAuth {
  public static final byte CONNECTION_ACCEPTED = -84;
  
  protected List<Class<?>> commandsAvailable;
  
  public PSAuth() {
    this.commandsAvailable = new LinkedList<>();
    this.commandsAvailable.add(PSCommandSetBaseChannel.class);
    this.commandsAvailable.add(PSCommandReadBlacklistedChannels.class);
    this.commandsAvailable.add(PSCommandReadMulticastGroups.class);
    this.commandsAvailable.add(PSCommandParameter.class);
    this.commandsAvailable.add(PSCommandSetSIB0and1.class);
    this.commandsAvailable.add(PSLogMessage.class);
  }
  
  public synchronized byte[] encryptMessage(byte[] message) {
    byte[] encrypted = null;
    try {
      byte[] gbiv = getKey();
      byte[] kb = Arrays.copyOfRange(gbiv, 0, 16);
      byte[] iv = Arrays.copyOfRange(gbiv, 16, 32);
      SecretKeySpec sk = new SecretKeySpec(kb, "AES");
      IvParameterSpec ivspec = new IvParameterSpec(iv);
      Cipher cipher = Cipher.getInstance("AES/CBC/NoPadding");
      cipher.init(1, sk, ivspec);
      encrypted = cipher.doFinal(message);
    } catch (NoSuchAlgorithmException|javax.crypto.NoSuchPaddingException e) {
      e.printStackTrace();
    } catch (InvalidKeyException e) {
      e.printStackTrace();
    } catch (InvalidAlgorithmParameterException e) {
      e.printStackTrace();
    } catch (IllegalBlockSizeException e) {
      e.printStackTrace();
    } catch (BadPaddingException e) {
      e.printStackTrace();
    } 
    return encrypted;
  }
  
  protected abstract byte[] getKey();
  
  public final boolean isCommandAvailable(Class<?> commandClass) {
    return (this.commandsAvailable.indexOf(commandClass) >= 0);
  }
}
