package ubiik.wp.qa.model;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import ubiik.utils.DatatypeConverter;
import ubiik.utils.communication.ProgressDelegate;
import ubiik.utils.model.ByteArrayUtils;
import ubiik.wp.ami.controller.UUTestResult;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.messages.PSDownlink;
import ubiik.wp.model.messages.PSMessage;

public class DownlinksStressTest extends PSMessageStressTest {
  private List<String> uuids;
  
  private boolean useRandomDevice;
  
  private int payloadSize;
  
  private int endpoint;
  
  private long dlCount;
  
  public DownlinksStressTest(ProgressDelegate progressDelegate, long millisInterval, Integer totalIterations, Long millisRuntime, MessageController messageController, List<String> uuids, int payloadSize, int endpoint) {
    super(progressDelegate, millisInterval, totalIterations, millisRuntime, messageController);
    this.useRandomDevice = (uuids == null);
    this.uuids = new ArrayList<>();
    if (!this.useRandomDevice)
      this.uuids.addAll(uuids); 
    this.payloadSize = payloadSize;
    this.endpoint = endpoint;
    this.dlCount = 0L;
  }
  
  public DownlinksStressTest(ProgressDelegate progressDelegate, long millisInterval, Integer totalIterations, Long millisRuntime, MessageController messageController, List<String> uuids, int payloadSize) {
    this(progressDelegate, millisInterval, totalIterations, millisRuntime, messageController, uuids, payloadSize, 1);
  }
  
  protected boolean runIteration(UUTestResult testResult, StringBuilder sb, long timeStart, Long millisRuntime, long millisInterval) {
    boolean success = true;
    boolean finished = false;
    if (this.useRandomDevice)
      this.uuids.add(DatatypeConverter.printHexBinary(
            ByteArrayUtils.getRandomApplicationData(16, null))); 
    for (int i = 0; i < this.uuids.size() && !finished && this.doRun.get(); i++) {
      PSDownlink dl = new PSDownlink(ByteArrayUtils.getRandomApplicationData(this.payloadSize, null), DatatypeConverter.parseHexBinary(this.uuids.get(i)), this.endpoint);
      this.messageController.sendMessage((PSMessage)dl);
      this.dlCount++;
      this.timeRunning = Calendar.getInstance().getTime().getTime() - timeStart;
      boolean connected = this.messageController.isConnected().get();
      success = (success && connected);
      if (!connected)
        sb.append("Lost connection in iteration ").append(this.totalIts)
          .append(" i: ").append(i)
          .append(" Downlink ").append(dl)
          .append(" After ").append(this.timeRunning).append("ms\n"); 
      if (this.millisRuntime != null && this.timeRunning > this.millisRuntime.longValue())
        finished = true; 
      try {
        Thread.sleep(this.millisInterval);
      } catch (InterruptedException e) {
        e.printStackTrace();
      } 
    } 
    if (this.useRandomDevice)
      this.uuids.remove(0); 
    return finished;
  }
  
  protected void setTestResult(StringBuilder sb) {
    this.testResult.setMessage("Ran " + this.totalIts + " iterations in " + this.timeRunning + "ms. Sent " + this.dlCount + " downlinks. Payload size: " + this.payloadSize + " interval: " + this.millisInterval + " ms");
  }
}
