package ubiik.wp.qa.model;

import java.util.Calendar;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import ubiik.utils.communication.ProgressDelegate;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.devices.UUEndDevice;
import ubiik.wp.model.messages.PSMessage;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandSetBaseChannel;

public class WARFCNTest extends BSTest implements PSCommandDelegate {
  private long timeout;
  
  private List<Integer> warfcns;
  
  private MessageController messageController;
  
  private DevicesController devicesController;
  
  private long timeRunning;
  
  public WARFCNTest(ProgressDelegate progressDelegate, MessageController messageController, DevicesController devicesController, long timeout, List<Integer> warfcns) {
    super(progressDelegate);
    this.timeout = timeout;
    this.warfcns = warfcns;
    this.messageController = messageController;
    this.devicesController = devicesController;
  }
  
  public void run() {
    this.doRun.set(true);
    Collection<UUEndDevice> initialDevices = this.devicesController.getDevices();
    this.timeRunning = 0L;
    long timeStart = Calendar.getInstance().getTime().getTime();
    boolean finished = false;
    int warfcnIdx = 0;
    int allReConnected = 0;//false
    setWarfcn(warfcnIdx);
    StringBuilder sb = new StringBuilder();
    while (this.doRun.get() && !finished) {
      int i = 0;
      try {
        Thread.sleep(1000L);
      } catch (InterruptedException e) {
        e.printStackTrace();
      } 
      this.timeRunning = Calendar.getInstance().getTime().getTime() - timeStart;
      allReConnected = 1;//true
      List<UUEndDevice> currentEds = new LinkedList<>(this.devicesController.getDevices());
      for (UUEndDevice ed : initialDevices) {
        int idx = currentEds.indexOf(ed);
        i = allReConnected & ((idx >= 0 && ((UUEndDevice)currentEds.get(idx)).getStatus() == 2) ? 1 : 0);
      } 
      if (i != 0) {
        sb.append("All devices rejoined the network for WARFCN: ").append(this.warfcns.get(warfcnIdx))
          .append(" after ").append(this.timeRunning).append("ms\n");
        warfcnIdx++;
        finished = !setWarfcn(warfcnIdx);
        this.timeRunning = 0L;
        timeStart = Calendar.getInstance().getTime().getTime();
        continue;
      } 
      if (this.timeRunning > this.timeout) {
        this.testResult.setSuccess(false);
        sb.append("Only ").append(this.devicesController.getDevices().size()).append(" of ")
          .append(initialDevices.size()).append(" rejoined the network for WARFCN: ")
          .append(this.warfcns.get(warfcnIdx)).append(" after ").append(this.timeRunning).append("ms\n");
        this.timeRunning = 0L;
        timeStart = Calendar.getInstance().getTime().getTime();
        warfcnIdx++;
        finished = !setWarfcn(warfcnIdx);
      } 
    } 
    this.testResult.setMessage(sb.toString());
    this.progressDelegate.progressFinished(null);
  }
  
  public boolean setWarfcn(int idx) {
    if (idx < this.warfcns.size()) {
      this.devicesController.clearDevices();
      this.messageController.sendMessage((PSMessage)new PSCommandSetBaseChannel(((Integer)this.warfcns
            .get(idx)).shortValue(), this));
      return true;
    } 
    return false;
  }
  
  public void commandandReceivedResponse(PSCommand command) {}
}
