package ubiik.wp.test;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Random;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.messages.pscommands.PSCommandFPGAReadRegisters;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponse;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseFPGAReadRegister;
import ubiik.wp.view.FPGARegisterViewController;
import ubiik.wp.view.FPGAViewController;

public class FPGASetFromCommandResultTest extends TestCase {
  private FPGAViewController fpgaController;

  private DevicesController dc;

  private MessageController messageController;

  @Rule
  public JavaFXThreadingRule javafxRule = new JavaFXThreadingRule();

  @Before
  protected void setUp() throws Exception {
    super.setUp();
    this.dc = new DevicesController();
    this.messageController = new MessageController(this.dc);
    FXMLLoader fpgaLoader = new FXMLLoader(getClass().getResource("/main/resources/ubiik/wp/admin/FPGAView.fxml"));
    try {
      this
        .fpgaController = new FPGAViewController(this.messageController.getDevicesController(), this.messageController, null);
      fpgaLoader.setController(this.fpgaController);
      Parent parent = fpgaLoader.<Parent>load();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  @After
  protected void tearDown() throws Exception {
    super.tearDown();
  }

  @Test
  public void testSetAllFromCommandResp() {
    int totalReg = this.fpgaController.getRegisterControllers().size();
    ByteBuffer commandData = ByteBuffer.allocate(2 + 4 * totalReg);
    commandData.put((byte)0);
    commandData.put((byte)-1);
    for (int i = 0; i < totalReg; i++) {
      int rnd = (new Random()).nextInt();
      commandData.putInt(rnd);
    }
    PSCommandResponseFPGAReadRegister response = new PSCommandResponseFPGAReadRegister(1, 1, commandData.array(), (short)9);
    PSCommandFPGAReadRegisters command = new PSCommandFPGAReadRegisters(this.dc.getFpgaRegisters().get(0), this.fpgaController.getRegisterControllers().get(0));
    command.setResponse((PSCommandResponse)response);
    commandData.position(0);
    assertEquals((byte)0, commandData.get());
    assertEquals((byte)-1, commandData.get());
    for (int j = 0; j < totalReg; j++) {
      int originalValue = commandData.getInt();
      int newValue = ((FPGARegisterViewController)this.fpgaController.getRegisterControllers().get(j)).getRegisterContent();
      assertEquals(originalValue, newValue);
    }
  }
}
