package ubiik.wp.test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import junit.framework.TestCase;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandBlacklistChannels;
import ubiik.wp.model.messages.pscommands.PSCommandRemoveBlacklistChannel;
import ubiik.wp.model.messages.pscommands.PSCommandSetBlacklistedChannels;

public class PSCommandBlacklistTest extends TestCase {
  @Before
  public void setUp() throws Exception {}
  
  @After
  public void tearDown() throws Exception {}
  
  public void doTestCommandBlacklistFormat(PSCommand cmd, byte[] expectedData, short expectedCmdType) {
    byte[] msgBytes = cmd.toBytes();
    ByteBuffer buffer = ByteBuffer.wrap(msgBytes).order(ByteOrder.LITTLE_ENDIAN);
    buffer.getShort();
    assertEquals((byte)4, buffer.get());
    buffer.getShort();
    assertEquals(expectedCmdType, buffer.getShort());
    byte[] cmdBytes = Arrays.copyOfRange(msgBytes, 7, msgBytes.length);
    Assert.assertArrayEquals(expectedData, cmdBytes);
  }
  
  @Test
  public void testSetBlacklistChannelsFormatNoChannels() {
    PSCommandSetBlacklistedChannels cmd = new PSCommandSetBlacklistedChannels(new int[0], null);
    byte[] expectedData = { 0, 0 };
    short expectedCmdType = 10;
    doTestCommandBlacklistFormat((PSCommand)cmd, expectedData, expectedCmdType);
  }
  
  @Test
  public void testBlacklistChannelsFormatNoChannels() {
    PSCommandBlacklistChannels cmd = new PSCommandBlacklistChannels(new int[0], null);
    byte[] expectedData = { 0, 0 };
    short expectedCmdType = 7;
    doTestCommandBlacklistFormat((PSCommand)cmd, expectedData, expectedCmdType);
  }
  
  @Test
  public void testRemoveBlacklistChannelsFormatNoChannels() {
    PSCommandRemoveBlacklistChannel cmd = new PSCommandRemoveBlacklistChannel(new int[0], null);
    byte[] expectedData = { 0, 0 };
    short expectedCmdType = 8;
    doTestCommandBlacklistFormat((PSCommand)cmd, expectedData, expectedCmdType);
  }
  
  @Test
  public void testSetBlacklistChannels() {
    int[] channels = { 2000, 2010, 2020, 2030 };
    PSCommandSetBlacklistedChannels cmd = new PSCommandSetBlacklistedChannels(channels, null);
    ByteBuffer buffer = ByteBuffer.allocate(2 + 2 * channels.length).order(ByteOrder.LITTLE_ENDIAN);
    buffer.putShort((short)channels.length);
    for (int i = 0; i < channels.length; i++)
      buffer.putShort((short)channels[i]); 
    byte[] expectedData = buffer.array();
    byte[] msgBytes = cmd.toBytes();
    byte[] cmdBytes = Arrays.copyOfRange(msgBytes, 7, msgBytes.length);
    Assert.assertArrayEquals(expectedData, cmdBytes);
  }
}
