package ubiik.wp.test;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import junit.framework.TestCase;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import ubiik.wp.model.messages.pscommands.PSCommandAddDevicesToMulticastGroup;
import ubiik.wp.model.messages.pscommands.PSCommandRemoveDevicesFromMulticastGroup;
import ubiik.wp.model.messages.pscommands.PSCommandSetMulticastGroups;

public class PSCommandMulticastTest extends TestCase {
  @Before
  public void setUp() throws Exception {}
  
  @After
  public void tearDown() throws Exception {}
  
  @Test
  public void testSetMulticastGroupsFormatNoGroups() {
    PSCommandSetMulticastGroups cmd = new PSCommandSetMulticastGroups(new HashMap<>(), null);
    byte[] msgBytes = cmd.toBytes();
    byte[] expectedData = { 0, 0 };
    ByteBuffer buffer = ByteBuffer.wrap(msgBytes).order(ByteOrder.LITTLE_ENDIAN);
    buffer.getShort();
    assertEquals((byte)4, buffer.get());
    buffer.getShort();
    assertEquals((short)6, buffer.getShort());
    byte[] cmdBytes = Arrays.copyOfRange(msgBytes, 7, msgBytes.length);
    Assert.assertArrayEquals(expectedData, cmdBytes);
  }
  
  @Test
  public void testSetMulticastGroupsOneGroup() {
    HashMap<Integer, List<byte[]>> multicastGroups = new LinkedHashMap<>();
    byte[] ed1 = { 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 5, 4, 3, 2, 1 };
    byte[] ed2 = { 
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
        2, 2, 2, 2, 2, 2 };
    List<byte[]> eds = (List)new ArrayList<>();
    eds.add(ed1);
    eds.add(ed2);
    Integer groupId = Integer.valueOf(128);
    multicastGroups.put(groupId, eds);
    PSCommandSetMulticastGroups cmd = new PSCommandSetMulticastGroups(multicastGroups, null);
    byte[] msgBytes = cmd.toBytes();
    byte[] expectedData = ByteBuffer.allocate(38).order(ByteOrder.LITTLE_ENDIAN).putShort((short)multicastGroups.size()).putShort(groupId.shortValue()).putShort((short)eds.size()).put(ed1).put(ed2).array();
    byte[] cmdBytes = Arrays.copyOfRange(msgBytes, 7, msgBytes.length);
    Assert.assertArrayEquals(expectedData, cmdBytes);
  }
  
  @Test
  public void testSetMulticastGroupsTwoGroups() {
    HashMap<Integer, List<byte[]>> multicastGroups = new LinkedHashMap<>();
    byte[] ed1 = { 
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
        0, 5, 4, 3, 2, 1 };
    byte[] ed2 = { 
        2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
        2, 2, 2, 2, 2, 2 };
    List<byte[]> eds = (List)new ArrayList<>();
    eds.add(ed1);
    eds.add(ed2);
    Integer groupId = Integer.valueOf(128);
    multicastGroups.put(groupId, eds);
    Integer groupId2 = Integer.valueOf(157);
    List<byte[]> eds2 = (List)new ArrayList<>();
    eds2.add(ed2);
    multicastGroups.put(groupId2, eds2);
    PSCommandSetMulticastGroups cmd = new PSCommandSetMulticastGroups(multicastGroups, null);
    byte[] msgBytes = cmd.toBytes();
    byte[] expectedData = ByteBuffer.allocate(58).order(ByteOrder.LITTLE_ENDIAN).putShort((short)multicastGroups.size()).putShort(groupId.shortValue()).putShort((short)eds.size()).put(ed1).put(ed2).putShort(groupId2.shortValue()).putShort((short)eds2.size()).put(ed2).array();
    byte[] cmdBytes = Arrays.copyOfRange(msgBytes, 7, msgBytes.length);
    Assert.assertArrayEquals(expectedData, cmdBytes);
  }
  
  @Test
  public void testAddMulticastGroupsFormatNoGroups() {
    PSCommandAddDevicesToMulticastGroup cmd = new PSCommandAddDevicesToMulticastGroup(new HashMap<>(), null);
    byte[] msgBytes = cmd.toBytes();
    byte[] expectedData = { 0, 0 };
    ByteBuffer buffer = ByteBuffer.wrap(msgBytes).order(ByteOrder.LITTLE_ENDIAN);
    buffer.getShort();
    assertEquals((byte)4, buffer.get());
    buffer.getShort();
    assertEquals((short)4, buffer.getShort());
    byte[] cmdBytes = Arrays.copyOfRange(msgBytes, 7, msgBytes.length);
    Assert.assertArrayEquals(expectedData, cmdBytes);
  }
  
  @Test
  public void testRemoveMulticastGroupsFormatNoGroups() {
    PSCommandRemoveDevicesFromMulticastGroup cmd = new PSCommandRemoveDevicesFromMulticastGroup(new HashMap<>(), null);
    byte[] msgBytes = cmd.toBytes();
    byte[] expectedData = { 0, 0 };
    ByteBuffer buffer = ByteBuffer.wrap(msgBytes).order(ByteOrder.LITTLE_ENDIAN);
    buffer.getShort();
    assertEquals((byte)4, buffer.get());
    buffer.getShort();
    assertEquals((short)5, buffer.getShort());
    byte[] cmdBytes = Arrays.copyOfRange(msgBytes, 7, msgBytes.length);
    Assert.assertArrayEquals(expectedData, cmdBytes);
  }
}
