package ubiik.wp.view;

import java.util.ArrayList;

import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.bsparameters.BSParameter;
import ubiik.wp.model.messages.PSMessage;
import ubiik.wp.model.messages.pscommands.PSCommand;
import ubiik.wp.model.messages.pscommands.PSCommandDelegate;
import ubiik.wp.model.messages.pscommands.PSCommandParameter;
import ubiik.wp.model.messages.pscommands.responses.PSCommandResponseBSParameter;

public class BSParameterViewController extends BaseViewController implements PSCommandDelegate {
  private int parameterIdx;
  
  @FXML
  private TextField input;
  
  @FXML
  private Label labelName;
  
  @FXML
  private Label labelValue;
  
  @FXML
  private Label labelMin;
  
  @FXML
  private Label labelMax;
  
  @FXML
  private Label labelDescription;
  
  public BSParameterViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver, int parameterIdx) {
    super(devicesController, messageController, commandStatusReceiver);
    this.parameterIdx = parameterIdx;
  }
  
  public void initialize() {
    BSParameter param = this.devicesController.getBSParameterForIdx(this.parameterIdx);
    if (param != null) {
          //goRAN ENBC
          //this.labelName.setText(param.getName());
          this.labelName.setText(param.getUiName());
          //goARN ENBC
          this.labelMin.setVisible(false);
          this.labelMax.setVisible(false);
          //this.labelMin.setText("Min: " + param.getMinStr());
          //this.labelMax.setText("Max: " + param.getMaxStr());
          this.labelValue.setText("Value: " + param.getValueString());
          this.labelDescription.setText(param.getDescription());
      
    } 
  }
  //Get response from ENBC
  public void commandandReceivedResponse(PSCommand command) {
    if (command.getResponse().isSuccess()) {
      setStatusBarCommandSuccess(command);
      if (command instanceof PSCommandParameter) {
        PSCommandParameter psCommandParameter = (PSCommandParameter)command;
        if (psCommandParameter.isSet()) {
          this.devicesController.setBSParameterValue(this.parameterIdx, psCommandParameter
              
              .getLongValue());
        } else if (psCommandParameter.isGet() && 
          command.getResponse() instanceof PSCommandResponseBSParameter) {
          PSCommandResponseBSParameter resp = (PSCommandResponseBSParameter)command.getResponse();
          this.devicesController.setBSParameterValue(this.parameterIdx, resp

              .getValue());
        } 
      } 
    } else {
      displayErrorMessage("Error", command.getTypeString() + " failed");
      setStatusBarCommandFailed(command);
    } 
  }
  
  public void buttonSetParameterPressed() {
    PSCommandParameter cmd = null;
    boolean valid = false;
    synchronized (this) {
      BSParameter paramToSend = new BSParameter(this.devicesController.getBSParameterForIdx(this.parameterIdx));
      if(paramToSend.getName().equals("S1_STATUS") || paramToSend.getName().equals("ENB_MODE"))
      {
        // Do nothing for these two commands.
        return;
      }
      Long value = null;
      try {
        value = Long.valueOf(Long.parseLong(this.input.getText()));
        valid = paramToSend.validateValue(value);
      } catch (Exception e) {
        valid = false;
      } 
      setTextFieldIsSuccessful(this.input, false);
      setTextFieldHasError(this.input, !valid);
      if (valid) {
        paramToSend.setValue(value);
        cmd = new PSCommandParameter(this, paramToSend, (byte)1);
        this.input.setText("");
      } 
    } 
    if (cmd != null)
      sendCommandAndUpdateStatusBar((PSCommand)cmd); 
  }
  
  public void buttonGetParameterPressed() {
    sendCommandGetParameter(true);
  }
  
  public PSCommandParameter sendCommandGetParameter(boolean displayInUI) {
    PSCommandParameter cmd = null;
    synchronized (this) {
      BSParameter paramToSend = new BSParameter(this.devicesController.getBSParameterForIdx(this.parameterIdx));
      paramToSend.setValue(null);
      cmd = new PSCommandParameter(this, paramToSend, (byte)0);
    } 
    if (cmd != null)
      if (displayInUI) {
        sendCommandAndUpdateStatusBar((PSCommand)cmd);
      } else {
        this.messageController.sendMessage((PSMessage)cmd);
      }  
    return cmd;
  }
  
  public PSCommandParameter getCommandGetParameter() {
    PSCommandParameter cmd = null;
    BSParameter paramToSend = new BSParameter(this.devicesController.getBSParameterForIdx(this.parameterIdx));
    paramToSend.setValue(null);
    cmd = new PSCommandParameter(this, paramToSend, (byte)0);
    return cmd;
  }
  
  public void updateInput() {
    BSParameter param = this.devicesController.getBSParameterForIdx(this.parameterIdx);
    if (param != null){
        if( param.getName().compareTo("ENB_MODE") == 0){
          if(param.getValueString().compareTo("0") == 0)
                setLabelText(this.labelValue, "LTE-M"); 
            else if(param.getValueString().compareTo("1") == 0)
                setLabelText(this.labelValue, "NB-IoT");
            else     
                setLabelText(this.labelValue, "N/A");  
        }
        else if( param.getName().compareTo("S1_STATUS") == 0){
            if(param.getValueString().compareTo("1") == 0)
                setLabelText(this.labelValue, "Connected"); 
            else
                setLabelText(this.labelValue, "Disconnected");
        }
         else if( param.getName().compareTo("S6_STATUS") == 0){
            if(param.getValueString().compareTo("1") == 0)
                setLabelText(this.labelValue, "Connected"); 
            else
                setLabelText(this.labelValue, "Disconnected");
        }
           
        else{
          if( param.getName().compareTo("BS_TX_PWR") == 0)
              setLabelText(this.labelValue,  param.getValueString() + " dBm"); 
          else
              setLabelText(this.labelValue,  param.getValueString());
        }     
    }
  }
  
 
   /* 
  public void updateInput() {
    
    ArrayList<BSParameter> parameters = this.devicesController.getBSParameters();
    for (int i = 0; i < parameters.size(); i++) {
        if(((BSParameter)parameters.get(i)).getName().compareTo("ENB_MODE") == 0 ){
              if(parameters.get(i).getValue() == 0)
                setLabelText(this.labelValue, "cat-M" );  
            else if(parameters.get(i).getValue() == 1)
                setLabelText(this.labelValue, "NB-IoT" );
             else  
                setLabelText(this.labelValue, "N/A" );             
        }
        else 
            setLabelText(this.labelValue, "Value: " + parameters.get(i).getValueString());
        }  
 

  }*/
  
  public void doReloadView() {}
}
