package ubiik.wp.view;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.TableCell;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.util.Callback;
import ubiik.utils.view.ColumnFormatter;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.MessageController;
import ubiik.wp.controller.PSMessageStorage;
import ubiik.wp.model.messages.pscommands.PSCommand;

public class CommandHistoryViewController extends BaseViewController {
  private static final int COMMANDS_IN_LIST = 500;
  
  @FXML
  private TableView<PSCommand> tableView;
  
  private final ObservableList<PSCommand> commands = FXCollections.observableArrayList();
  
  public CommandHistoryViewController(DevicesController devicesController, MessageController messageController, CommandStatusReceiver commandStatusReceiver) {
    super(devicesController, messageController, commandStatusReceiver);
  }
  
  public void initialize() {
    this.tableView.getColumns().clear();
    TableColumn<PSCommand, Integer> idCol = new TableColumn<>("ID");
    TableColumn<PSCommand, String> typeCol = new TableColumn<>("Command");
    TableColumn<PSCommand, String> contentCol = new TableColumn<>("Content");
    TableColumn<PSCommand, String> statusCol = new TableColumn<>("Status");
    TableColumn<PSCommand, Date> datelCol = new TableColumn<>("Date");
    idCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.1D));
    typeCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.25D));
    contentCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.3D));
    statusCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.15D));
    datelCol.prefWidthProperty().bind(this.tableView.widthProperty().multiply(0.2D));
    idCol.setSortable(true);
    typeCol.setSortable(true);
    contentCol.setSortable(false);
    statusCol.setSortable(false);
    datelCol.setSortable(true);
    idCol.setCellValueFactory(new PropertyValueFactory<>("applicationId"));
    typeCol.setCellValueFactory(new PropertyValueFactory<>("typeString"));
    contentCol.setCellValueFactory(new PropertyValueFactory<>("contentString"));
    statusCol.setCellValueFactory(new PropertyValueFactory<>("statusStr"));
    datelCol.setCellValueFactory(new PropertyValueFactory<>("timestamp"));
    datelCol.setCellFactory((Callback<TableColumn<PSCommand, Date>, TableCell<PSCommand, Date>>)new ColumnFormatter(DisplayConstants.SIMPLE_DATE_FORMAT_TIME_DATE));
    this.tableView.getColumns().addAll(new TableColumn[] { idCol, typeCol, contentCol, statusCol, datelCol });
    this.tableView.setItems(this.commands);
    reloadView();
  }
  
  public void doReloadView() {
    synchronized (this.commands) {
      PSCommand selectedCmd = this.tableView.getSelectionModel().getSelectedItem();
      PSMessageStorage ms = PSMessageStorage.getInstance();
      List<PSCommand> cmds = ms.getCommands(500);
      this.commands.clear();
      this.commands.addAll(cmds);
      Collections.reverse(this.commands);
      this.tableView.sort();
      this.tableView.getSelectionModel().select(selectedCmd);
    } 
  }
}
