package ubiik.wp.view;

import java.nio.ByteBuffer;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.css.PseudoClass;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import ubiik.utils.DatatypeConverter;
import ubiik.wp.model.fpga.FPGARegisterField;

public class FPGARegisterFieldController {
  @FXML
  private Label labelName;
  
  @FXML
  private Label labelMsb;
  
  @FXML
  private Label labelLsb;
  
  @FXML
  private TextField inputValue;
  
  @FXML
  private Label labelDefault;
  
  @FXML
  private Label labelWrite;
  
  @FXML
  private Label labelDescription;
  
  private FPGARegisterField registerField;
  
  public FPGARegisterFieldController(FPGARegisterField registerField) {
    this.registerField = registerField;
  }
  
  public void initialize() {
    this.labelName.setText(this.registerField.getName());
    this.labelLsb.setText(
        Integer.toString(this.registerField.getLsb()));
    this.labelMsb.setText(
        Integer.toString(this.registerField.getMsb()));
    this.inputValue.setText(
        getTextForValue(this.registerField.getValue()));
    this.inputValue.setEditable(this.registerField.isWritable());
    this.inputValue.setDisable(!this.registerField.isWritable());
    this.labelDefault.setText(
        getTextForValue(this.registerField.getDefaultValue()));
    String str = this.registerField.isWritable() ? "R/W" : "R";
    this.labelWrite.setText(str);
    this.labelDescription.setText(this.registerField.getDescription());
    this.inputValue.focusedProperty().addListener(new ChangeListener<Boolean>() {
          public void changed(ObservableValue<? extends Boolean> arg0, Boolean oldPropertyValue, Boolean newPropertyValue) {
            if (!newPropertyValue.booleanValue())
              FPGARegisterFieldController.this.validateInput(true); 
          }
        });
  }
  
  private String getTextForValue(int value) {
    ByteBuffer buffer = ByteBuffer.allocate(4);
    buffer.putInt(value);
    String str = DatatypeConverter.printHexBinary(buffer.array());
    int charForBytes = this.registerField.getMsb() - this.registerField.getLsb() + 1 + 3 >> 2;
    return str.substring(str.length() - charForBytes, str.length());
  }
  
  public boolean validateInput(boolean save) {
    boolean valid = false;
    int intValue = 0;
    String valueStr = this.inputValue.getText();
    if (valueStr.length() > 0 && valueStr.length() <= 8) {
      if (valueStr.length() < 8)
        valueStr = (new String(new char[8 - valueStr.length()])).replace("\000", "0") + (new String(new char[8 - valueStr.length()])).replace("\000", "0"); 
      try {
        intValue = Integer.parseUnsignedInt(valueStr, 16);
        valid = this.registerField.validateValue(intValue);
        if (valid)
          this.registerField.setValue(intValue); 
      } catch (Exception e) {
        valid = false;
      } 
    } 
    setFieldWithError(!valid);
    return valid;
  }
  
  private void setFieldWithError(boolean error) {
    PseudoClass errorClass = PseudoClass.getPseudoClass("error");
    this.inputValue.pseudoClassStateChanged(errorClass, error);
  }
  
  public void setFrom32Bit(int value) {
    if (this.registerField.isWritable())
      this.registerField.setFrom32Bit(value); 
    setFieldWithError(false);
    this.inputValue.setText(getTextForValue(this.registerField.getValue()));
  }
}
