package ubiik.wp.view;

import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.scene.Parent;
import javafx.scene.control.Button;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import ubiik.wp.controller.BaseStationNameManager;
import ubiik.wp.controller.DevicesController;
import ubiik.wp.controller.EndDeviceNameAndIdxManager;
import ubiik.wp.controller.MessageController;
import ubiik.wp.model.devices.DevicesStorage;
import ubiik.wp.model.devices.MulticastGroup;
import ubiik.wp.model.devices.UUEndDevice;
import ubiik.wp.model.messages.pscommands.PSCommandATCommand;
import ubiik.wp.model.messages.pscommands.PSCommandFPGAReadRegisters;
import ubiik.wp.model.messages.pscommands.PSCommandSDRSet;
import ubiik.wp.model.messages.pscommands.PSCommandSetSIB0and1;

public class MainViewController extends WPMainViewController implements CommandStatusReceiver {
  private static final String DEFAULT_BS_NAMES_CSV = "bs_names.csv";

  private static final String DEFAULT_ED_NAMES_CSV = "ed_names.csv";

  @FXML
  private Button fpgaConfigButton;

  @FXML
  private Button configButton;

  @FXML
  private Button sdrConfigButton;

  @FXML
  private Button buttonSIB;

  @FXML
  private Button buttonAtCommand;

  @FXML
  private VBox buttonsMenu;

  @FXML
  protected VBox noConnectionButtonsMenu;

  private FPGAViewController fpgaController;

  private Parent fpgaPane;

  private UplinksViewController uplinksController;

  private Parent uplinksPane;

  protected DownlinksViewController downlinksController;

  protected Parent downlinksPane;

  private DevicesViewController devicesViewController;

  private Parent devicesPane;

  private MulticastGroupsViewController multicastViewController;

  private Parent multicastPane;

  protected ConfigurationViewController configController;

  protected Parent configPane;

  private CommandHistoryViewController commandHistoryController;

  private Parent commandHistoryPane;

  private SIBViewController sibViewController;

  private Parent sibPane;

  private SDRViewController sdrViewController;

  private Parent sdrPane;

  private ATCommandViewController atCommandViewController;

  private Parent atCommandPane;

  private TestModeViewController testModeViewController;

  private Parent testModePane;

  private DeviceViewController deviceViewController;

  private Parent devicePane;

  protected AppSettingsViewController settingsViewController;

  protected Parent settingsPane;

  private FrequencyScannerViewController frequencyScanViewController;

  private Parent frequencyScanPane;

  private MulticastGroupViewController multicastGroupViewController = null;

  private Parent multicastGroupPane;

  private MCSViewController mcsViewController;

  private Parent mcsPane;

  private CapacityDownlinkViewController capacityViewController;

  private Parent capacityPane;

  private CapacityUplinkViewController capacityUplinkViewController;

  private Parent capacityUplinkPane;

  protected Stage primaryStage;
  
  private Integer numEndDevices;
  
  private Integer receviedProcess;

  public MainViewController(Stage primaryStage, MessageController mc, DevicesController dc) {
    super(mc, dc);
    this.fpgaController = null;
    this.fpgaPane = null;
    this.uplinksController = null;
    this.uplinksPane = null;
    this.downlinksController = null;
    this.downlinksPane = null;
    this.configController = null;
    this.configPane = null;
    this.sibViewController = null;
    this.sibPane = null;
    this.sdrViewController = null;
    this.sdrPane = null;
    this.atCommandViewController = null;
    this.atCommandPane = null;
    this.settingsViewController = null;
    this.settingsPane = null;
    this.capacityViewController = null;
    this.capacityPane = null;
    this.capacityUplinkViewController = null;
    this.capacityUplinkPane = null;
    this.primaryStage = primaryStage;
    this.numEndDevices = -1;
    this.receviedProcess = 0;
    primaryStage.setTitle("Ubiik Config Tool");
  }

  public void initialize() {
    super.initialize();
    BaseStationNameManager.getInstance().loadNamesFromPath("bs_names.csv");
    EndDeviceNameAndIdxManager edIdxManager = EndDeviceNameAndIdxManager.getInstance();
    edIdxManager.loadNamesFromPath("ed_names.csv");
    if (edIdxManager.getMaxIndex() >= 0)
      this.devicesController.setIdxForEds(edIdxManager.getDevicesAndIndexes(), edIdxManager.getMaxIndex(), true);
    if (!this.user.isCommandAvailable(PSCommandFPGAReadRegisters.class))
      this.buttonsMenu.getChildren().remove(this.fpgaConfigButton);
    if (!this.user.isCommandAvailable(PSCommandSDRSet.class))
      this.buttonsMenu.getChildren().remove(this.sdrConfigButton);
    if (!this.user.isCommandAvailable(PSCommandSetSIB0and1.class))
      this.buttonsMenu.getChildren().remove(this.buttonSIB);
    if (!this.user.isCommandAvailable(PSCommandSDRSet.class))
      this.buttonsMenu.getChildren().remove(this.sdrConfigButton);
    if (!this.user.isCommandAvailable(PSCommandATCommand.class))
      this.buttonsMenu.getChildren().remove(this.buttonAtCommand);
    this.labelStatus.setText("");
    Platform.runLater(new Runnable() {
          public void run() {
            MainViewController.this.displayConfig();
            MainViewController.this.loadSibViewController();
            MainViewController.this.disableMenuAndActions(true);
          }
        });
  }

  public void authenticationFinished(boolean autheticated) {
    super.authenticationFinished(autheticated);
    if (autheticated) {
      Platform.runLater(new Runnable() {
            public void run() {
              MainViewController.this.displayConfig();
            }
          });
    }
  }

  public void disableMenuAndActions(boolean disable) {
    this.buttonsMenu.setDisable(disable);
    if (this.rootPane != null && this.rootPane.getCenter() != null)
      this.rootPane.getCenter().setDisable(disable);
  }

  @FXML
  public void uplinksButtonPressed() {
    if (this.uplinksController == null)
      this.uplinksController = new UplinksViewController();
    this.uplinksPane = setCurrentView(this.uplinksController, this.uplinksPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/UplinksView.fxml"));
  }

  @FXML
  public void fpgaButtonPressed() {
    if (this.fpgaController == null)
      this
        .fpgaController = new FPGAViewController(this.messageController.getDevicesController(), this.messageController, this);
    this.fpgaPane = setCurrentView(this.fpgaController, this.fpgaPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/FPGAView.fxml"));
  }

  @FXML
  public void devicesButtonPressed() {
    if (this.devicesViewController == null)
      this
        .devicesViewController = new DevicesViewController(this.messageController.getDevicesController(), this.messageController, this, this);
    this.devicesPane = setCurrentView(this.devicesViewController, this.devicesPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/DevicesView.fxml"));
  }

  @FXML
  public void multicastGroupsButtonPressed() {
    if (this.multicastViewController == null)
      this
        .multicastViewController = new MulticastGroupsViewController(this.primaryStage, this.messageController.getDevicesController(), this.messageController, this, this);
    this.multicastPane = setCurrentView(this.multicastViewController, this.multicastPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/MulticastGroupsView.fxml"));
  }

  protected void displayConfig() {
    if (this.configController == null)
      this
        .configController = new ConfigurationViewController(this.messageController.getDevicesController(), this.messageController, this);
    this.configPane = setCurrentView(this.configController, this.configPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/ConfigurationView.fxml"));
  }

  @FXML
  public void buttonConfigPressed() {
    displayConfig();
  }

  @FXML
  public void sdrButtonPressed() {
    if (this.sdrViewController == null)
      this
        .sdrViewController = new SDRViewController(this.messageController.getDevicesController(), this.messageController, this);
    this.sdrPane = setCurrentView(this.sdrViewController, this.sdrPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/SDRView.fxml"));
  }

  @FXML
  public void buttonDownlinksPressed() {
    if (this.downlinksController == null)
      this
        .downlinksController = new DownlinksViewController(this.messageController.getDevicesController(), this.messageController, this, true);
    this.downlinksPane = setCurrentView(this.downlinksController, this.downlinksPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/DownlinksViewController.fxml"));
  }

  @FXML
  public void buttonCommandHistoryPressed() {
    if (this.commandHistoryController == null)
      this
        .commandHistoryController = new CommandHistoryViewController(this.messageController.getDevicesController(), this.messageController, this);
    this.commandHistoryPane = setCurrentView(this.commandHistoryController, this.commandHistoryPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/CommandHistoryView.fxml"));
  }

  private void loadSibViewController() {
    if (this.sibViewController == null)
      this
        .sibViewController = new SIBViewController(this.messageController.getDevicesController(), this.messageController, this);
  }

  @FXML
  public void buttonSIBPressed() {
    loadSibViewController();
    this.sibPane = setCurrentView(this.sibViewController, this.sibPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/SIBView.fxml"));
  }

  @FXML
  public void buttonAtCommandPressed() {
    if (this.atCommandViewController == null)
      this
        .atCommandViewController = new ATCommandViewController(this.messageController.getDevicesController(), this.messageController, this);
    this.atCommandPane = setCurrentView(this.atCommandViewController, this.atCommandPane, MainViewController.class.getResource("/main/java/ubiik/wp/view/ATCommandView.fxml"));
  }

  @FXML
  public void buttonTestModesPressed() {
    if (this.testModeViewController == null)
      this
        .testModeViewController = new TestModeViewController(this.messageController.getDevicesController(), this.messageController, this);
    this.testModePane = setCurrentView(this.testModeViewController, this.testModePane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/TerminalView.fxml"));
  }

  public void buttonSettingsPressed() {
    if (this.settingsViewController == null)
      this.settingsViewController = new AppSettingsViewController(this.primaryStage, this, this.devicesController, this.messageController, this);
    this.settingsPane = setCurrentView(this.settingsViewController, this.settingsPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/SettingsView.fxml"));
  }

  public void buttonFrequencyScanPressed() {
    if (this.frequencyScanViewController == null)
      this.frequencyScanViewController = new FrequencyScannerViewController(this.devicesController, this.messageController, this);
    this.frequencyScanPane = setCurrentView(this.frequencyScanViewController, this.frequencyScanPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/FrequencyScannerView.fxml"));
  }

  public void buttonMCSPressed() {
    if (this.mcsViewController == null)
      this.mcsViewController = new MCSViewController(this.devicesController, this.messageController, this);
    this.mcsPane = setCurrentView(this.mcsViewController, this.mcsPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/mcsView.fxml"));
  }

  public void buttonCapacityTestPressed() {
    if (this.capacityViewController == null)
      this.capacityViewController = new CapacityDownlinkViewController(this.primaryStage, this.devicesController, this.messageController, this);
    this.capacityPane = setCurrentView(this.capacityViewController, this.capacityPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/CapacityView.fxml"));
  }

  public void buttonCapacityUplinkTestPressed() {
    if (this.capacityUplinkViewController == null)
      this.capacityUplinkViewController = new CapacityUplinkViewController(this.primaryStage, this.devicesController, this.messageController, this);
    this.capacityUplinkPane = setCurrentView(this.capacityUplinkViewController, this.capacityUplinkPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/CapacityUplinkView.fxml"));
  }

  public void displayEndDeviceView(UUEndDevice uuEndDevice) {
    this.deviceViewController = null;
    this.devicePane = null;
    this.deviceViewController = new DeviceViewController(this.devicesController, this.messageController, this, uuEndDevice);
    this.devicePane = setCurrentView(this.deviceViewController, this.devicePane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/DeviceView.fxml"));
  }

  public void displayMulticastGroupView(MulticastGroup mcg) {
    this.multicastGroupViewController = null;
    this.multicastGroupPane = null;
    this.multicastGroupViewController = new MulticastGroupViewController(this.devicesController, this.messageController, this, mcg);
    this.multicastGroupPane = setCurrentView(this.multicastGroupViewController, this.multicastGroupPane, MainViewController.class.getResource("/main/resources/ubiik/wp/admin/MulticastGroupView.fxml"));
  }

  public void setLastStatus(String lastStatus) {
    setLastStatusLabel(lastStatus);
  }

  public boolean getBSInfo () {
	  this.receviedProcess = DevicesStorage.getInstance().getDevices().size();
	  this.numEndDevices = this.devicesController.getNumEndDevices();

	  if ((this.receviedProcess >= this.numEndDevices) && 
		  (this.numEndDevices >= 0)) 
	  {
		  if (this.configController != null)
	        this.configController.readStatus();
	      if (this.sibViewController != null)
	        this.sibViewController.readSIB();
	      if (this.devicesController.isInactiveDevicesMonitoringEnabled())
	        this.devicesController.startTrackingInactiveDevices(this.messageController);
	      return true;
	  }
	  return false;
  }
  
  public void doReloadView() {}
}
