package ubiik.wp.view;

import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.atomic.AtomicBoolean;
import javafx.application.Platform;

public abstract class RefreshableViewController {
  public static final int DEFAULT_REFRESH_FREQUENCY = 200;

  private Timer timeOutTimerTimer;

  private static int refreshFrequency = 200;

  private static Object refreshFrequencyLock = new Object();

  private static AtomicBoolean refreshEnabled = new AtomicBoolean(true);

  private AtomicBoolean isRefreshing = new AtomicBoolean(false);

  public final void reloadView() {
    if (!this.isRefreshing.get()) {
      this.isRefreshing.set(true);
      Platform.runLater(new Runnable() {
            public void run() {
              RefreshableViewController.this.doReloadView();
              RefreshableViewController.this.isRefreshing.set(false);
            }
          });
    }
  }

  private void reloadAndSetTimer() {
    if (refreshEnabled.get())
      reloadView();
    TimerTask reloadTask = new TimerTask() {
        public void run() {
          RefreshableViewController.this.reloadAndSetTimer();
        }
      };
    if (this.timeOutTimerTimer != null)
      this.timeOutTimerTimer.cancel();
    this.timeOutTimerTimer = new Timer(true);
    synchronized (refreshFrequencyLock) {
      this.timeOutTimerTimer.schedule(reloadTask, refreshFrequency);
    }
  }

  public void onShow() {
    reloadAndSetTimer();
  }

  public void onHide() {
    if (this.timeOutTimerTimer != null)
      this.timeOutTimerTimer.cancel();
  }

  public static int getRefreshFrequency() {
    return refreshFrequency;
  }

  public static void setRefreshFrequency(int freq) {
    synchronized (refreshFrequencyLock) {
      refreshFrequency = freq;
    }
  }

  public static void setRefreshEnabled(boolean enable) {
    refreshEnabled.set(enable);
  }

  public abstract void doReloadView();
}
